///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2008
// 
// Filename: CHAM_PIC_TEST_PGM_VER1.c
//
// Original Author: Joshua Hintze
// 
// Last Modified: 8.28.09
//
// Description: Test utility for the chameleon
//				
//
// Required Files: CHAM_PIC_SYSTEM_V010.c/h
//				   CHAM_PIC_I2C_SPI_DRV_V010.c/h
//				   CHAM_PIC_UART_DRV_V010.c/h
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>


// Include main system header
#include "CHAM_PIC_SYSTEM_V010.h"

// Include spi driver
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"

// Include the UART driver
#include "CHAM_PIC_UART_DRV_V010.h"

// Include some api drivers
#include "CHAM_PIC_NTSC_DRV_V010.h"
#include "CHAM_PIC_VGA_DRV_V010.h"
#include "CHAM_PIC_MOUSE_DRV_V010.h"
#include "CHAM_PIC_FLASH_DRV_V010.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#define IsNum(ch) 		(ch >= 30 && ch <= 39)
#define IsAlpha(ch) 	((ch >= 65 && ch <= 90) || (ch >= 97 && ch <= 122))
#define IsAlphaUpr(ch)	(ch >= 65 && ch <= 90)
#define IsAlphaLwr(ch)	(ch >= 97 && ch <= 122)
#define IsAlphaNum(ch) 	{IsNum(ch) || IsAlpha(ch))
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// String buffer for reading in from the terminal
char sbuffer[80];

// The following is DTMF codes used in a touch tone telephone system
// Numbers			0    1    2    3    4    5    6    7    8    9
int DTMFLow[10] = { 941, 697, 697, 697, 770, 770, 770, 852, 852, 852 } ;
int DTMFHigh[10]= {1336,1209,1336,1477,1209,1336,1477,1209,1336,1477 } ;


// we could use PGM memory for these and or do it algorithmically, but this is cleaner for now
// 11 sectors [64K, 64K, 64K, 64K, 64K, 64K, 64K, 32K, 8K, 8k, 16K ]
//            |<------- (7) 64K sectors ------->|  1      2     1 

#define NUM_FLASH_SECTORS_4MEGS 11

long sector_addresses_4Megs[] = {   0x00000,
		                            0x10000,
		                            0x20000,
		                            0x30000,
		                            0x40000,
		                            0x50000,
		                            0x60000,
		                            0x70000,
		                            0x78000,
		                            0x7A000,
		                            0x7D000 };

// we could use PGM memory for these and or do it algorithmically, but this is cleaner for now
// 19 sectors [64K, 64K, 64K, ..., 64K, 64K, 64K, 64K, 16K, 8K, 8k, 32K ]
//            |<----------- (15) 64K sectors -------->| 1     2     1 

#define NUM_FLASH_SECTORS_8MEGS 19

long sector_addresses_8Megs[] = {   0x00000,   // 64K sector 0 
		                            0x10000,   // 64K sector 1
		                            0x20000,   // 64K sector 2 
		                            0x30000,   // 64K sector 3
		                            0x40000,   // 64K sector 4
		                            0x50000,   // 64K sector 5 
		                            0x60000,   // 64K sector 6
		                            0x70000,   // 64K sector 7 
		                            0x80000,   // 64K sector 8
		                            0x90000,   // 64K sector 9
		                            0xA0000,   // 64K sector 10
		                            0xB0000,   // 64K sector 11
		                            0xC0000,   // 64K sector 12
		                            0xD0000,   // 64K sector 13
		                            0xE0000,   // 64K sector 14
		
		                            0xF0000,   // 16K sector 15
		                            0xF4000,   //  8K sector 16
		                            0xF6000,   //  8K sector 17
		
		                            0xF8000 }; // 32K sector 18

// flash string for testing
char *flash_string = "Atari 800 Rules Forever!!!";


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void _strlwr(char *CmdStr);
void _strupr(char *CmdStr);
long _atol2(char *Str);
int atoi2(char *Str);




// Our main start location
int main (void)
{
	unsigned char RXValue;
	int NumTokens = 0;
	char CmdStr[40];				// Allow 40 bytes for our input buffer.
	char TokenStr[40];				// Copy of CmdStr so when strtok mangles it we'll have original
	int CmdStrAddPtr = 0;
	char *Tokens[10];				// Up to 10 Tokens
	char *TokenPtr;
	long arg0, arg1, arg2, arg3, arg4, arg5; // generic args for command processor
	int  next_token = 0;
	int  num_Tokens = 0;
	unsigned char spi_data = 0;
	long index;

	int flash_passed, flash_attempt;       // used for flash test

	long flash_start_addr, flash_value, flash_count;
    gid_event event;                                  // generic "event" for mouse, etc.
	int x=16, y=12;                   // dummy vars for testing...

	int num_flash_sectors = NUM_FLASH_ROM_SECTORS;


	// Always call ConfigureClock first
	SYS_ConfigureClock(MAX_FCY_RATE);

	// Initialize the UART interface
	UART_Init(115200);

	// Initialize the spi interface
	SPI_SET_CS(SPI_CS_0);
	SPI_Init();

	UART_vt100ClearScreen();
	UART_printf("\n\rChameleon PIC Test Console Rev A. (c) 2009 Nurve Networks LLC\n\r");
	UART_printf("\r\nREADY>");



	while(1)
	{
		if(UART_getchar(&RXValue))
		{
			// Add character to our current string if its not a control code
			if(RXValue >= 32 && RXValue <= 126)
			{
				// Add it to our buffer
				CmdStr[CmdStrAddPtr++] = (char)RXValue;

				// Echo value back
				UART_putchar(RXValue);
			}
			else
			{
				if(RXValue == 127 || RXValue == 8)	// Is Backspace?
				{
					if(CmdStrAddPtr)				// Only allow backspace if there is data in the CmdStr
					{
						CmdStrAddPtr--;
						CmdStr[CmdStrAddPtr] = 0;
						UART_putchar(127);			// Send 127 because it looks better than backspace
					}
				}
				else if(RXValue == '\r') 			// Carriage return
				{
					CmdStr[CmdStrAddPtr++] = 0; 	// Null terminate

					// lower case string for comparision
					_strupr(CmdStr);

					// Copy over the string before it gets destroyed by strtok
					memcpy(TokenStr, CmdStr, CmdStrAddPtr);

					// Tokenize string up to 10 values
					NumTokens = 0;

					// Get first token
					TokenPtr = strtok(TokenStr, " ,=");

					while(TokenPtr)
					{
						// Input token into token array and increment number of Tokens
						Tokens[NumTokens++] = TokenPtr;
						TokenPtr = strtok(NULL, " ,=");
					}

					//uart_printf("\r\nNumTokens = %d\r\n",NumTokens);
					//for(i = 0; i < NumTokens; i++)
					//	uart_printf("Tokens(%d) = %s\r\n",i,Tokens[i]);

					// Now determine what kind of command the user just entered
					if(!strcmp("HELP", Tokens[0]))
					{
						UART_printf("\r\n\r\nList of available commands:\r\n");
						UART_printf("_____________________________________________________________________________\r\n");
						UART_printf("help       | Displays this help screen\r\n");
						UART_printf("clear      | (cls) Clears the screen screen\r\n");
						UART_printf("snd (.play, .stop, .stopall) | Plays and stops tones\r\n");
						UART_printf("\r\n");
						UART_printf("flash (fl)   | Test the flas interface\r\n");
						UART_printf("     .WE      | Write enable\r\n");
						UART_printf("     .WD      | Write disable\r\n");
						UART_printf("     .RS      | Read status\r\n");
						UART_printf("     .RD start count | Read status\r\n");
						UART_printf("     .SP      | Sequential program\r\n");
						UART_printf("     .BP      | Byte program\r\n");
						UART_printf("     .BE4     | Block erase 4\r\n");
						UART_printf("     .CE      | Chip erase\r\n");
						UART_printf("     .JI      | Jedec ID\r\n");
						UART_printf("     .US      | Unprotect sector\r\n");
						UART_printf("     .UC      | Unprotect chip\r\n");
						UART_printf("\r\n");
						
					}


// TEST FIXTURE TO TEST API

               else
               if ( !strcmp( Tokens[0], "TEST") )
                  {
                  // enter handler

                //UART_printf("\r\nTesting FLASH Memory...\r\n");

                // make n attempts to pass FLASH memory
                for (flash_attempt = 0; flash_attempt < 3; flash_attempt++)
                    {                 
                    sprintf(sbuffer,"\r\nTesting FLASH memory");
                    UART_printf( sbuffer );                            
                    for (index = 0; index <= flash_attempt; index++)
                      UART_putchar( '.' );                              
                   	
	                // STEP 1: enable writing to chip  ////////////////////////////////////////////////////////
	                // enable SPI interface
	   
	                // set CS to SPI select channel 3 (FLASH)
	                SPI_SET_CS(SPI_CS_FLASH);
	
	                UART_printf("\r\nExec Write Enable...");
	                spi_data = SPI_WriteRead( WRITE_ENABLE );
	
	                // disable SPI interface
	   
	                // set CS to SPI select channel 0 (null)
	                SPI_SET_CS(SPI_CS_0);
	                DELAY_MS( 100 );
	
	                // STEP 1A: erase first 4K block of chip, in case of any garbage
	
	                flash_start_addr = 0; // default to block 0
	
	                // enable SPI interface
	
	                // set CS to SPI select channel 3 (FLASH)
	                SPI_SET_CS(SPI_CS_FLASH);
	
	                UART_printf("\r\nExec block erase 4K Cmd...");
	                spi_data = SPI_WriteRead( BLOCK_ERASE4 );
	
	                // write block address, there are only a handful of blocks since each is 4-64K
	                spi_data = SPI_WriteRead( flash_start_addr >> 16 );
	                spi_data = SPI_WriteRead( flash_start_addr >> 8 );
	                spi_data = SPI_WriteRead( flash_start_addr >> 0 );
	
	                // disable SPI interface
	                // set CS to SPI select channel 0 (null)
	                SPI_SET_CS(SPI_CS_0);
	
	                // erasure takes a long time... 
	                DELAY_MS( 500 );
	
	                // STEP 1B: enable writing to the chip again
	                // enable SPI interface
	   
	                // set CS to SPI select channel 3 (FLASH)
	                SPI_SET_CS(SPI_CS_FLASH);
	                UART_printf("\r\nExec Write Enable...");
	                spi_data = SPI_WriteRead( WRITE_ENABLE );
	
	                DELAY_MS( 100 );
	
	                // STEP 2: unprotect sector starting at address 0 //////////////////////////////////////////
	                flash_start_addr = 0; // default to sector 0
	
	                // enable SPI interface
	
	                // set CS to SPI select channel 3 (FLASH)
	                SPI_SET_CS(SPI_CS_FLASH);
	
	                UART_printf("\r\nExec Unprotect Sec Cmd...");
	                spi_data = SPI_WriteRead( UNPROTECT_SECTOR );
	
	                // write sector address, there are only a handful of sectors since each is 4-64K
	                spi_data = SPI_WriteRead( flash_start_addr >> 16 );
	                spi_data = SPI_WriteRead( flash_start_addr >> 8 );
	                spi_data = SPI_WriteRead( flash_start_addr >> 0 );
	
	                // disable SPI interface
	                // set CS to SPI select channel 0 (null)
	                SPI_SET_CS(SPI_CS_0);
	
	                DELAY_MS( 100 );
	
	                // STEP 3: enable writing to chip again /////////////////////////////////////////////////////
	                // enable SPI interface
	   
	                // set CS to SPI select channel 3 (FLASH)
	                SPI_SET_CS(SPI_CS_FLASH);
	
	                UART_printf("\r\nExec Write Enable...");
	                spi_data = SPI_WriteRead( WRITE_ENABLE );
	
	                // disable SPI interface
	   
	                // set CS to SPI select channel 0 (null)
	                SPI_SET_CS(SPI_CS_0);
	
	                DELAY_MS( 100 );
	
	                // STEP 4: write test string starting at absolute address 0x00000 in flash //////////////////
	
	
	                 // set CS to SPI select channel 3 (FLASH)
	                 SPI_SET_CS(SPI_CS_FLASH);
	
	                 UART_printf("\r\nExec Seq Pgm...");
	
	                 flash_start_addr = 0;
	                 flash_count      = strlen( flash_string );
	                 flash_value      = flash_string[ 0 ];
	
	
	                 sprintf(sbuffer,"\r\n%ld %ld %s",flash_start_addr,
	                                                flash_count,
	                                                flash_string);
	
	
	                 UART_printf( "%s", sbuffer );              
	            
	
	                 // initiate seq program command
	                 spi_data = SPI_WriteRead( SEQ_PROGRAM );
	
	                 // write starting address byte by byte, MSB first...
	                 spi_data = SPI_WriteRead( flash_start_addr >> 16 );
	                 spi_data = SPI_WriteRead( flash_start_addr >> 8 );
	                 spi_data = SPI_WriteRead( flash_start_addr >> 0 );
	
	                 sprintf(sbuffer,"\r\nWrt string '%s' from [%ld..%ld]",flash_string, 
	                                                                     flash_start_addr, 
	                                                                     flash_start_addr + flash_count-1);
	                                                             
	                 UART_printf( "%s", sbuffer );
	
	                 // write first byte
	                 spi_data = SPI_WriteRead( flash_string[ 0 ] );
	
	                 // de-assert set CS to SPI select channel 0 (null)
	                 SPI_SET_CS(SPI_CS_0);
	                 
	                 // now write remaining data
	                 for (index = 1; index < flash_count; index++)
	                     {
	
	                     // set CS to SPI select channel 3 (FLASH)
	                     SPI_SET_CS(SPI_CS_FLASH);
	          
	                     // re-issue seq program command, but NO address needed
	                     spi_data = SPI_WriteRead( SEQ_PROGRAM );
	                     // now data
	                     spi_data = SPI_WriteRead( flash_string[ index ] );
	
	                     // set CS to SPI select channel 0 (null)
	                     SPI_SET_CS(SPI_CS_0);
	
	                     // progress bar..    
	                     UART_putchar('.');
	
	                     } // end for index
	
	                // disable SPI interface
	                
	                // set CS to SPI select channel 0 (null)
	                SPI_SET_CS(SPI_CS_0);
	
	                // finally send write disable command
	
	                // set CS to SPI select channel 3 (FLASH)
	                SPI_SET_CS(SPI_CS_FLASH);
	                
	                spi_data = SPI_WriteRead( WRITE_DISABLE );
	                
	                // set CS to SPI select channel 0 (null)
	                SPI_SET_CS(SPI_CS_0);
	
	                // we have written the data to the flash, now read it back and confirm its the same
	                DELAY_MS(1);
	
	
	                 // set CS to SPI select channel 3 (FLASH)
	                 SPI_SET_CS(SPI_CS_FLASH);
	
	                 UART_printf("\r\nExec Read Byte(s)...");
	
	                 flash_start_addr = 0;
	                 flash_count      = strlen( flash_string );
	             
	                 // initiate read command
	                 spi_data = SPI_WriteRead( READ_DATA );
	
	                 // write starting address byte by byte, MSB first...
	                 spi_data = SPI_WriteRead( flash_start_addr >> 16 );
	                 spi_data = SPI_WriteRead( flash_start_addr >> 8 );
	                 spi_data = SPI_WriteRead( flash_start_addr >> 0 );
	
	                 // set flas passed flag
	                 flash_passed = 1;
	
	                 //strcpy(flash_string, "Atari 800 Rules Forever!!!" );
	
	                 // now read data 
	                 for (index = 0; index < flash_count; index++)
	                     {
	                     flash_value = SPI_WriteRead( 0x00 );
	
	                     sprintf(sbuffer,"%c,",(char)flash_value);
	                     UART_printf( "%s", sbuffer );                            
	
	                     // test the data
	                     if (flash_value != flash_string[ index ])
	                        {
	                        sprintf(sbuffer,"\r\nflash data doesn't match pattern at location [%ld] = '%c', should be '%c'.",index, (char)flash_value, (char)flash_string[ index ]);
	                        UART_printf( "%s", sbuffer );                            
	                        flash_passed = 0;
	                        break;
	                        
	                        } // there was an error 
	
	                     } // end for index
	
	                // disable SPI interface
	
	                // set CS to SPI select channel 0 (null)
	                SPI_SET_CS(SPI_CS_0);

	                // now compare buffer to flash_string???? 
	                if (flash_passed)
	                   {
	                   // exit for loop
	                   flash_attempt = 3;
	                   break;
	                   } // end if
	           
                	} // end for flash_attempt 
	
				  // now compare buffer to flash_string???? 
				  if (flash_passed)
				     UART_printf( "\r\n\r\nFLASH Memory Test: PASSED." );                            
				  else
				     UART_printf( "\r\n\r\nFLASH Memory Test: FAILED." );                            
                 

                  UART_printf("\r\n\r\nPress <ENTER> to Continue PS/2 Mouse Test.");
                  while(UART_getchar(&RXValue)==0);

                  // load the mouse
                  Mouse_Load();

                  // clear screens
                  NTSC_ClearScreen();
                  NTSC_Color(0);
                  NTSC_Term_Print(" PS/2 Mouse/SPI/Color Test");

                  VGA_ClearScreen();
                  VGA_Color(0);
                  VGA_Term_Print(" PS/2 Mouse/SPI/Color Test");

                  NTSC_SetXY( 1, 2 );
                  VGA_SetXY( 1, 2 );

                  // print colored numbers 0123456
                  for (index = 0; index < 7; index++)
                    {
                    NTSC_Color((int)index);
                    NTSC_Term_Char('0'+(int)index);

                    VGA_Color((int)index);
                    VGA_Term_Char('0'+(int)index);
                    } // end for index

                  UART_printf("\r\nMouse Test Running -- Make sure PS/2 Mouse is plugged into Chameleon PS/2 port." );
                  UART_printf("\r\nTesting Mouse & SPI Interace, press <ENTER> to Exit...");
               
                  // main event loop
                  while(UART_getchar(&RXValue)==0)
                       {
                       Mouse_Read(&event); 

                       event.x = event.x / 10;
                       event.y = event.y / 10;

                       // button hit?
                       if (event.buttons == 0X01)
                         {
                         // clear the surface
                         NTSC_ClearScreen();
                         VGA_ClearScreen();
                         } // end if

                       // motion clamping                  
                       if (event.x < -2)
                         event.x = -2;
                       else 
                       if (event.x > 2)
                           event.x = 2;

                       if (event.y < -2)
                         event.y = -2;
                       else 
                       if (event.y > 2)
                           event.y = 2;

                       // update position
                       x += event.x;
                       y -= event.y;

                       // draw on both screens
                       NTSC_SetXY( x, y );
                       NTSC_Term_Char('X');

                       VGA_SetXY( x, y/2 );
                       VGA_Term_Char('X');

                       // output to serial console on PC what's going on
                       // sprintf(sbuffer,"\r\nMouse x=%d, y=%d, z=%d, buttons=%X | Press <ESC> to stop test.", x, y, event.z, event.buttons );
                       // UART_Print_String( sbuffer );                            

                       // give everything a 
                       DELAY_MS( 5 );
                       } // end while

                  UART_printf("\r\n");


                  } // end if


            /*
            NTSC Commands - These commands are processed and issued to NTSC terminal core.
           
            ntsc .pr [string]      - This command prints a single string token to the NTSC terminal. 
            ntsc .cls              - Clear and home the NTSC terminal.
            ntsc .col [num 0..7]   - Set color of NTSC terminal
            ntsc .nl               - Print a newline to NTSC terminal.
            ntsc .out [num 0..255] - Outputs a single ASCII character to NTSC terminal.
            ntsc .sx [num 0..39]   - Sets X cursor position of NTSC terminal.
            ntsc .sy [num 0..29]   - Sets Y cursor position of NTSC terminal.
            ntsc .gx               - Gets X cursor position of NTSC terminal.
            ntsc .gy               - Gets Y cursor position of NTSC terminal.


            .OUT - Output a character supports these sub-commands
            
                 $00 = clear screen
                 $01 = home
                 $08 = backspace
                 $09 = tab (8 spaces per)
                 $0A = set X position (X follows)
                 $0B = set Y position (Y follows)
                 $0C = set color (color follows)
                 $0D = carriage return 
           */

            else
            if ( !strcmp( Tokens[0], "NTSC" ) )
               {
              // case .PR "print"
              if ( !strcmp( Tokens[1], ".PR" ))                                    
                 {

                 SPI_Prop_Print_String( DEVICE_NTSC, Tokens[ 2 ] );
                 SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, 0x0D, 0x00);
                 } // end if
                    
              // case .CLS "clear screen", note we could implement this command directly thru the OUT character
              // command, but then we can only use the out() function within the graphics driver, breaking the "clearscreen"
              // out as a command, allows us to do some extra processing on both sides of the SPI driver, locally and in the 
              // Prop SPI driver interface, in essence we are overloading the command to add some functionality to it, or in 
              // this case just doing it for fun as an example. 
              else
              if ( !strcmp( Tokens[1], ".CLS" ) )
                 {
                 // clear screen                                     
                 SPI_Prop_Send_Cmd( GFX_CMD_NTSC_CLS, 0x00, 0x00);  

                 } // end if

              //  case .OUT "output a single character"
              else
              if ( !strcmp( Tokens[1], ".OUT" ) || !strcmp( Tokens[1], ".O" ) )                                    
                 {
                 arg1 = atoi2( Tokens[2] ) % 256;
                 SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, arg1, 0x00);
                 } // end if

              //  case .NL "newline"
              else
              if ( !strcmp( Tokens[1], ".NL" ) )                                    
                {
                SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, '\r', 0x00);
                } // end if

              // case .COL "set color"
              else
              if ( !strcmp( Tokens[1], ".COL" ) )
                 {
                 // extract color 0..7
                 arg1 = atoi2( Tokens[ 2 ]) % 8;
                
                 // send driver set color command, [$0C, col 0..7]                           
                 SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, 0x0C, 0x00);
                 SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, arg1, 0x00);
                 } // end if

              // case .SX "set cursor x position"
              else
              if ( !strcmp( Tokens[1], ".SX") )
                {
                // extract x
                arg1 = atoi2( Tokens[ 2 ]) % 40;

                // send driver set x command, [$0A, x 0..39]           
                SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, 0x0A, 0x00);
                SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, arg1, 0x00);
                } // end if

              // case .SY "set cursor y position"
              else
              if ( !strcmp( Tokens[1], ".SY" ) )
                 {
                 // extract y
                 arg1 = atoi2( Tokens[ 2 ]) % 30;

                 // send driver set x command, [$0B, y 0..29]                           
               
                 SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, 0x0B, 0x00);
                 SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, arg1, 0x00);
                 } // end if

              // case .GX "get cursor x position"
              else
              if ( !strcmp( Tokens[1], ".GX") )
                {
                // send driver get x command
                SPI_Prop_Send_Cmd( GFX_CMD_NTSC_GETX, 0x00, 0x00);

				// Give it some time to queue the request
				DELAY_US(200);

                // now read the data back, note 0x00, 0x00 are dummy data, mask lower 8-bits
                arg1 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00) & 0xFF;

                UART_printf( "\nX-Pos / Col = %d", (int)arg1 );

                } // end if

              // case .GY "get cursor x position"
              else
              if ( !strcmp( Tokens[1], ".GY") )
                {
                // send driver get y command
                SPI_Prop_Send_Cmd( GFX_CMD_NTSC_GETY, 0x00, 0x00);

				// Give it some time to queue the request
				DELAY_US(200);

                // now read the data back, note 0x00, 0x00 are dummy data, mask lower 8-bits
                arg1 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00) & 0xFF;

                UART_printf( "\nY-Pos / Row = %d", (int)arg1 );

                } // end if

				UART_printf("\r\n");

               } // end ntsc case


			/*
            VGA Commands - These commands are processed and issued to VGA terminal core.
           
            vga .pr [string]      - This command prints a single string token to the VGA terminal. 
            vga .cls              - Clear and home the VGA terminal.
            vga .col [num 0..7]   - Set color of VGA terminal
            vga .nl               - Print a newline to VGA terminal.
            vga .out [num 0..255] - Outputs a single ASCII character to VGA terminal.
            vga .sx [num 0..39]   - Sets X cursor position of VGA terminal.
            vga .sy [num 0..29]   - Sets Y cursor position of VGA terminal.
            vga .gx               - Gets X cursor position of VGA terminal.
            vga .gy               - Gets Y cursor position of VGA terminal.


            .OUT - Output a character supports these sub-commands
            
                 $00 = clear screen
                 $01 = home
                 $08 = backspace
                 $09 = tab (8 spaces per)
                 $0A = set X position (X follows)
                 $0B = set Y position (Y follows)
                 $0C = set color (color follows)
                 $0D = carriage return 
            */


            else
            if ( !strcmp( Tokens[0], "VGA" ) )
               {
              // case .PR "print"
              if ( !strcmp( Tokens[1], ".PR" ) || !strcmp( Tokens[1], ".P" ))                                    
                 {

                 SPI_Prop_Print_String( DEVICE_VGA, Tokens[ 2 ] );
                 SPI_Prop_Send_Cmd( GFX_CMD_VGA_PRINTCHAR, 0x0D, 0x00);
                 } // end if
                    
              // case .CLS "clear screen", note we could implement this command directly thru the OUT character
              // command, but then we can only use the out() function within the graphics driver, breaking the "clearscreen"
              // out as a command, allows us to do some extra processing on both sides of the SPI driver, locally and in the 
              // Prop SPI driver interface, in essence we are overloading the command to add some functionality to it, or in 
              // this case just doing it for fun as an example. 
              else
              if ( !strcmp( Tokens[1], ".CLS" ) )
                 {
                 // clear screen                                     
                 SPI_Prop_Send_Cmd( GFX_CMD_VGA_CLS, 0x00, 0x00);  
                 } // end if

              //  case .OUT "output a single character"
              else
              if ( !strcmp( Tokens[1], ".OUT" ) || !strcmp( Tokens[1], ".O" ) )                                    
                 {
                 arg1 = atoi2( Tokens[2] ) % 256;
                 SPI_Prop_Send_Cmd( GFX_CMD_VGA_PRINTCHAR, arg1, 0x00);
                 } // end if

              //  case .NL "newline"
              else
              if ( !strcmp( Tokens[1], ".NL" ) )                                    
                {
                SPI_Prop_Send_Cmd( GFX_CMD_VGA_PRINTCHAR, '\r', 0x00);
                } // end if

              // case .COL "set color"
              else
              if ( !strcmp( Tokens[1], ".COL" ) )
                 {
                 // extract color 0..7
                 arg1 = atoi2( Tokens[ 2 ]) % 8;
                
                 // send driver set color command, [$0C, col 0..7]                           
                 SPI_Prop_Send_Cmd( GFX_CMD_VGA_PRINTCHAR, 0x0C, 0x00);
                 SPI_Prop_Send_Cmd( GFX_CMD_VGA_PRINTCHAR, arg1, 0x00);
                 } // end if

              // case .SX "set cursor x position"
              else
              if ( !strcmp( Tokens[1], ".SX") )
                {
                // extract x
                arg1 = atoi2( Tokens[ 2 ]) % 40;

                // send driver set x command, [$0A, x 0..39]           
                SPI_Prop_Send_Cmd( GFX_CMD_VGA_PRINTCHAR, 0x0A, 0x00);
                SPI_Prop_Send_Cmd( GFX_CMD_VGA_PRINTCHAR, arg1, 0x00);
                } // end if

              // case .SY "set cursor y position"
              else
              if ( !strcmp( Tokens[1], ".SY" ) )
                 {
                 // extract y
                 arg1 = atoi2( Tokens[ 2 ]) % 30;

                 // send driver set x command, [$0B, y 0..29]                           
               
                 SPI_Prop_Send_Cmd( GFX_CMD_VGA_PRINTCHAR, 0x0B, 0x00);
                 SPI_Prop_Send_Cmd( GFX_CMD_VGA_PRINTCHAR, arg1, 0x00);
                 } // end if

              // case .GX "get cursor x position"
              else
              if ( !strcmp( Tokens[1], ".GX") )
                {
                // send driver get x command
                SPI_Prop_Send_Cmd( GFX_CMD_VGA_GETX, 0x00, 0x00);

				// Give it some time to queue the request
				DELAY_US(200);

                // now read the data back, note 0x00, 0x00 are dummy data, mask lower 8-bits
                arg1 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00) & 0xFF;

                UART_printf( "\nX-Pos / Col = %d", (int)arg1 );

                } // end if

              // case .GY "get cursor x position"
              else
              if ( !strcmp( Tokens[1], ".GY") )
                {
                // send driver get y command
                SPI_Prop_Send_Cmd( GFX_CMD_VGA_GETY, 0x00, 0x00);

				// Give it some time to queue the request
				DELAY_US(200);

                // now read the data back, note 0x00, 0x00 are dummy data, mask lower 8-bits
                arg1 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00) & 0xFF;

                UART_printf( "\nY-Pos / Row = %d", (int)arg1 );

                } // end if

				UART_printf("\r\n");

               } // end vga case


            /*
            Used to test GFX commands with "gfx" version 2 of Chameleon driver
            GFX GPU command

            GFX .W reg value
            GFX .R reg

            */

            else
            if ( !strcmp( Tokens[0], "GFX" ) )
               {
              // case .W "write register"
              if ( !strcmp( Tokens[1], ".W" ))                                    
                 {
                 // extract 8-bit register to write
                 arg1 = atoi2( Tokens[2] );

                 // extract 8/16-bit value to write
                 arg2 = atoi2( Tokens[3] );
              
                 // send write command: cmd, low, high
                 SPI_Prop_Send_Cmd( GPU_GFX_BASE_ID + arg1, (arg2 & 0xFF), (arg2 >> 8));
                 } // end if
                
              // case .R "read register"
              else
              if ( !strcmp( Tokens[1], ".R" ))                                    
                 {
              
                 // extract 8-bit register to write
                 arg1 = atoi2( Tokens[2] );
              
                 // send read command: cmd, 0,0
                 SPI_Prop_Send_Cmd( GPU_GFX_BASE_ID + arg1, 0x00, 0x00);
    
                 // let prop get data ready
                 DELAY_MS(10);
                       
                 // now retrieve results, note, if we read the results back too soon, the driver might not have time to 
                 // set them, so keep that in mind
                 arg2 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);
             
                  
                 // print results
                 UART_printf( "\r\nGFX GPU Register data = [%d]", (int)arg2 );        

                 } // end if

				UART_printf("\r\n");
                 
              } // end if GFX GPU command



	              else if ( !strcmp( Tokens[0], "KBD") )
                  {

					 if ( !strcmp( Tokens[1], ".READ"))
                     { 
	                   // read the keyboard on the slave side and print to NTSC terminal until
	                   // user hits a key on console
	
						while(!UART_getchar(&RXValue))
						{
							// request keyboard read from slave
							SPI_Prop_Send_Cmd(  KEY_CMD_KEY, 0x00, 0x00); 
							DELAY_US(250);
	
							// keyboard should have enough time to be read now, request key by performing general read back command
							// spi system will send back data in "result" buffer 
							
							if ((arg2 = SPI_Prop_Send_Cmd(  READ_CMD, 0x00, 0x00))!=0)
							{
								arg2 = arg2 & 0xFF;
								UART_printf("key = %c\r\n", (char)(arg2-32));
							}
							
							DELAY_MS( 25 );
						
						} // end while
					} // end if read

	 				else if ( !strcmp( Tokens[1], ".START")) 
	                   {
	                   SPI_Prop_Send_Cmd(  KEY_CMD_START, 0x00, 0x00);
	                   } // end if .START
	                else if ( !strcmp( Tokens[1], ".STOP")) 
	                   {
	                   SPI_Prop_Send_Cmd(  KEY_CMD_STOP, 0x00, 0x00);
	                   } // end if .STOP
	                else if ( !strcmp( Tokens[1], ".PRESENT")) 
	                   {
	                   SPI_Prop_Send_Cmd(  KEY_CMD_PRESENT, 0x00, 0x00);
	                   
	                   DELAY_MS(1);
	
	                   // now read result
	                   arg1 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);
	                   
	                   UART_printf( "Present = %d\r\n", (int)arg1);
	
	                   } // end if .PRESENT

					UART_printf("\r\n");
	
	              } // end if keyboard


					/*
					propeller local 8-bit port I/O commands
					
					PORT .D [0..255] - sets the 8-bit I/O pin directions for the port 1=output, 0=input
					PORT .R          - reads the 8-bit port pins, outputs are don't cares, so you must ignore them and mask off
					PORT .W [0..255] - writes the 8-bit port pins, port pins set to input ignore data and won't get to pins
					
					*/


					
					else if ( !strcmp( Tokens[0], "PORT" ) )
					{
						// case .W "write port"
						if ( !strcmp( Tokens[1], ".W" ))                                    
						{
							// extract 8-bit value to write
							arg1 = atoi2( Tokens[2] ) % 256;
					
							// send write port command to prop
							SPI_Prop_Send_Cmd( PORT_CMD_WRITE, arg1, 0x00);
						} // end if
					
						// case .R "read port"
						else if ( !strcmp( Tokens[1], ".R" ))                                    
						{
							// print the port out until user hits local terminal key
							while(!UART_getchar(&RXValue))
							{
								// send read port command to prop
								SPI_Prop_Send_Cmd( PORT_CMD_READ, 0x00, 0x00);
					
								// now retrieve results, note, if we read the results back too soon, the driver might not have time to 
								// set them, so keep that in mind
								arg2 = SPI_Prop_Send_Cmd(  READ_CMD, 0x00, 0x00);
					
								// only need lower 8-bits of result, this is 8-bit byte port only
								arg2 = arg2 & 0xFF;
					
								// print results
								sprintf(sbuffer, "\r\nPropeller port data = [%d]", (int)arg2);
								UART_printf( sbuffer );
							} // end while
						} // end if
						// case .D "set direction of port bits"
						else if ( !strcmp( Tokens[1], ".D" ))                                    
						{
							// extract 8-bit value to write as direction, 1=output, 0=input, 11110000 = out, out, out, out, in, in, in, in
							arg1 = atoi2( Tokens[2] ) % 256;
					
							// send setdir port command to prop
							SPI_Prop_Send_Cmd( PORT_CMD_SETDIR, arg1, 0x00);
						} // end if

						UART_printf( "\r\n" );
					
					} // end if port command

					else if(!strcmp("CLEAR", Tokens[0]) || !strcmp("CLS", Tokens[0]))
					{
						UART_vt100ClearScreen();
						UART_vt100SetCursor(0, 0);
					}
	                else if ( !strcmp( Tokens[0], "SOUND") || !strcmp( Tokens[0], "SND"))
	                {
 						// case .PLAY
                  		if (!strcmp( Tokens[1], ".PLAY" ) )
						{
		                      // SND [channel 0..3] [freq 0..2K] [ duration 0..7 ]
		                      // enter handler
		                      
		                      // TODO - add some error handling to this!
		
		                      // channel 0..3
		                      arg1 = atoi2( Tokens[ 2 ] );
		
		                      // frequency to play
		                      arg2 = atoi2( Tokens[ 3 ] );
		
		                      // duration
		                      arg3 = atoi2( Tokens[ 4 ] );
		
		                      // extract lower 2-bits of channel, shift to upper 2-bits of byte, then extract lower 3-bits of duration,
		                      // shift behind channel, then finally OR in upper 3-bits of frequency
		                      arg4 = (arg1 << 6) | (arg3 << 3) | (arg2 >> 8);
		
		                      // mask off upper 6-bits of frequency and send only lower 8-bits 
		                      arg5 = (arg2 & 0xFF);
		
		                      // final data format
		                      // arg4 [cc.dddfff ] | arg5 [ ffffffff ]
		                      // channel 2-bit (0..3) | duration 3-bit 0..7 | frequency 11-bit (0 turn off channel)
		                      SPI_Prop_Send_Cmd( SND_CMD_PLAYSOUNDFM, (int)arg5, (int)arg4);
						}
						                  // case .STOP
						else if ( !strcmp( Tokens[1], ".STOP" ) )                                  
						{				
							// [channel 0..3]
							// enter handler
							
							// channel 0..3
							arg1 = ( atoi2( Tokens[ 2 ] ) % 4);
							
							// issue command to sound core
							if (arg1 >= 0)
								SPI_Prop_Send_Cmd( SND_CMD_STOPSOUND, (int)arg1, 0);
						
						} // end if .STOP
												
						// case .STOPALL
						else if ( !strcmp( Tokens[1], ".STOPALL" ) )                                    
							SPI_Prop_Send_Cmd( SND_CMD_STOPALLSOUNDS, 0, 0);
	     

						UART_printf("\r\n");

                   } // end if sound



               else if ( !strcmp( Tokens[0], "MOUSE") )
               {
                   // read the mouse on the slave side and print to UART terminal until
                   // user hits a key on console

                  if ( !strcmp( Tokens[1], ".ABS"))
                     { 
                    	while(!UART_getchar(&RXValue))
                           {
                           // request mouse read from slave, driver takes time to respond...

                           // read absolute x position first
                           SPI_Prop_Send_Cmd( MOUSE_CMD_ABS_X, 0x00, 0x00); 
                           DELAY_US(500);
    
                           // now read data from SPI buffer
                           arg1 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00); 
                           DELAY_US(500);

                           // read absolute y position 
                           SPI_Prop_Send_Cmd( MOUSE_CMD_ABS_Y, 0x00, 0x00); 
                           DELAY_US(500);
    
                           // now read data from SPI buffer
                           arg2 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00); 
                           DELAY_US(500);

                           // read absolute z position 
                           SPI_Prop_Send_Cmd( MOUSE_CMD_ABS_Z, 0x00, 0x00); 
                           DELAY_US(500);
    
                           // now read data from SPI buffer
                           arg3 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00); 
                           DELAY_US(500);

                           // print results                          
                           UART_printf( "ABS[X, Y, Z] = [%d, %d, %d]\r\n", (int)arg1, (int)arg2, (int)arg3 );

                           DELAY_MS(25);
                        
                           } // end while

                  } // end if .ABS
                  else if ( !strcmp( Tokens[1], ".DELTA"))
                     { 
                  	 while(!UART_getchar(&RXValue))
                           {
                           // request mouse read from slave, driver takes time to respond...

                           // read absolute x position first
                           SPI_Prop_Send_Cmd( MOUSE_CMD_DELTA_X, 0x00, 0x00); 
                           DELAY_US(500);
    
                           // now read data from SPI buffer
                           arg1 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00) & 0xFF; 
                           DELAY_US(500);


                           // read absolute y position 
                           SPI_Prop_Send_Cmd( MOUSE_CMD_DELTA_Y, 0x00, 0x00); 
                           DELAY_US(500);
    
                           // now read data from SPI buffer
                           arg2 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00) & 0xFF; 
                           DELAY_US(500);


                           // read absolute z position 
                           SPI_Prop_Send_Cmd( MOUSE_CMD_DELTA_Z, 0x00, 0x00); 
                           DELAY_US(500);
    
                           // now read data from SPI buffer
                           arg3 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00)  & 0xFF; 
                           DELAY_US(500);


                           // print results							
                           UART_printf( "DELTA[X, Y, Z] = [%d, %d, %d]\r\n", (signed char)arg1, (signed char)arg2, (signed char)arg3 );

                           DELAY_MS(25);
                        
                           } // end while

                    } // end if .DELTA

                else if ( !strcmp( Tokens[1], ".START")) 
                   {
                   SPI_Prop_Send_Cmd(  MOUSE_CMD_START, 0x00, 0x00);
                   } // end if .START
                else if ( !strcmp( Tokens[1], ".STOP")) 
                   {
                   SPI_Prop_Send_Cmd(  MOUSE_CMD_STOP, 0x00, 0x00);
                   } // end if .STOP
                else if ( !strcmp( Tokens[1], ".PRESENT")) 
                   {
                   SPI_Prop_Send_Cmd(  MOUSE_CMD_PRESENT, 0x00, 0x00);
                   
                   DELAY_MS(1);

                   // now read result
                   arg1 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00) & 0xFF;
                   
                   UART_printf( "Present = %d\r\n", (int)arg1); 

                   } // end if .PRESENT

					UART_printf("\r\n");

                  } // end if read mouse





					/*
					propeller COG 0 (SPI driver COG) register access commands - BEWARE!
					
					REG .W32 [0..15]        - performs a 32-bit write to the addressed register [0..F] from the output register buffer         
					REG .R32 [0..15]        - performs a 32-bit read from the addressed register [0..F] and stores in the input register buffer       
					REG .WB [0..3] [0..255] - write byte 0..3 of output register g_reg_out_buffer.byte[  0..3  ]         
					REG .RB [0..3]          - read byte 0..3 of inout register g_reg_in_buffer.byte[  0..3  ]         
					*/
					
					else if ( !strcmp( Tokens[0], "REG" ) )
					{
						// case .WCOG "write COG SFR register from 32-bit output buffer"
						if ( !strcmp( Tokens[1], ".W32" ))                                    
						{
							// performs a 32-bit write to the addressed register [0..F] from the output register buffer         
					
							// extract register 0..15 to write 32-bit output buffer to
							arg1 = atoi2( Tokens[2] ) % 16; // registers 0..15
					
							// send write COG register to prop
							SPI_Prop_Send_Cmd( REG_CMD_WRITE, arg1, 0x00);
						} // end if
						else if ( !strcmp( Tokens[1], ".R32" ))                                    
						// case .RCOG "reads COG SFR register into 32-bit input buffer"
						{
							// performs a 32-bit read from the addressed register [0..F] and stores in the input register buffer       
					
							// extract register 0..15 to read into 32-bit input buffer
							arg1 = atoi2( Tokens[2] ) % 16; // registers 0..15
					
							// send read COG register to prop
							SPI_Prop_Send_Cmd( REG_CMD_READ, arg1, 0x00);
						} // end if
						else if ( !strcmp( Tokens[1], ".WB" ))                                    
						// case .WB "write register byte buffer register 0..3"
						{
							// write byte 0..3 of output register g_reg_out_buffer.byte[  0..3  ]         
					
							// extract byte to write 0..3 to in 32-bit output buffer
							arg1 = atoi2( Tokens[2] ) % 4; // byte buffer register 0..3
					
							// extract 8-bit data to write 
							arg2 = atoi2( Tokens[3] ) % 256; 
					
							// send write byte register command to prop
							SPI_Prop_Send_Cmd( REG_CMD_WRITE_BYTE, arg1, arg2);
						} // end if
						else if ( !strcmp( Tokens[1], ".RB" ))                                    
						// case .RB "read register byte buffer register 0..3"
						{
							// read byte 0..3 of input register g_reg_in_buffer.byte[  0..3 ]
							// this data is then placed into spi buffer for transport back to client                 
					
							// extract byte to read 0..3 from in 32-bit input buffer
							arg1 = atoi2( Tokens[2] ) % 4; // byte buffer register 0..3
					
							// send read byte register command to prop
							SPI_Prop_Send_Cmd( REG_CMD_READ_BYTE, arg1, 0x00);
					
							// now that data has been read into SPI buffer, pull it out
							DELAY_CLOCKS(10000);
							
							// now retrieve results, note, if we read the results back too soon, the driver might not have time to 
							// set them, so keep that in mind
							arg2 = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);
							
							// only need lower 8-bits of result
							arg2 = arg2 & 0xFF;
							
							// print results
							sprintf(sbuffer, "\nByte[%d] = %d", (int)arg1, (int)arg2);
							UART_printf( sbuffer );			
						} // end if
					} // end if REGister command

	               else
	               if ( !strcmp( Tokens[0], "RESET") )
	                  {
	                  // allows the PIC, Prop or both to be reset
	                 if (!strcmp( Tokens[1], ".PIC") )  
	                    {
	                    // reset the PIC
	                    //PIC_Soft_Reset();
	
	                    } // end if
	                 else
	                 if (!strcmp( Tokens[1], ".PROP") )  
	                    {
	                    // reset the prop
	                    SPI_Prop_Send_Cmd( SYS_RESET, 0x00, 0x00);
	                    } // end if
	                 else
	                 if (!strcmp( Tokens[1], ".ALL") )  
	                    {
	                    // reset the prop
	                    SPI_Prop_Send_Cmd( SYS_RESET, 0x00, 0x00);
	
	                    // reset the PIC
	                    //PIC_Soft_Reset();
	                  
	                    } // end if

						UART_printf("\r\n");
	
	                  } // end if RESET

	               else
	               if ( !strcmp( Tokens[0], "FLASH") || !strcmp( Tokens[0], "FL"))
	                  {
	                  // enter handler
	                  arg0 = 0; // null command    
	                  next_token = 0;
	                                   
	                  // determine which command is being requested, parse first parameter, search for "cmd=" or a number
	
	                  if (!strcmp( Tokens[1], ".CMD"))
	                     {
	                     // user is entering command directly with a number in hex or decimal format
	                     arg0 = _atol2( Tokens[2] );
	                     next_token = 3;
	                     
	                     //sprintf(sbuffer, "parsed: CMD = %d", arg0);                    
	                     //UART_printf( sbuffer );
	
	                     } // end if  
	                  else if (!strcmp( Tokens[1], ".WE"))  {arg0 = WRITE_ENABLE;  next_token = 2; }
	                  else if (!strcmp( Tokens[1], ".WD")) {arg0 = WRITE_DISABLE; next_token = 2; }
	                  else if (!strcmp( Tokens[1], ".RS") || !strcmp( Tokens[1], "STATUS"))   {arg0 = READ_STATUS;   next_token = 2; }    
	                  //else if (!strcmp( Tokens[1], "WRITE_STATUS"))  {arg0 = WRITE_STATUS;  next_token = 2; }   
	                  else if (!strcmp( Tokens[1], ".RD"))     {arg0 = READ_DATA;     next_token = 2; }   
	                  else if (!strcmp( Tokens[1], ".SP") )  {arg0 = SEQ_PROGRAM;  next_token = 2; }   
	                  else if (!strcmp( Tokens[1], ".BP") )  {arg0 = BYTE_PROGRAM;  next_token = 2; }   
	                  else if (!strcmp( Tokens[1], ".BE4"))   {arg0 = BLOCK_ERASE4;   next_token = 2; }   
	                  //else if (!strcmp( Tokens[1], "SECTOR_ERASE"))  {arg0 = SECTOR_ERASE;  next_token = 2; }       
	                  else if (!strcmp( Tokens[1], ".CE"))    {arg0 = CHIP_ERASE;    next_token = 2; }
	                  else if (!strcmp( Tokens[1], ".JI"))      {arg0 = JEDEC_ID;      next_token = 2; }
	                  else if (!strcmp( Tokens[1], ".US"))  {arg0 = UNPROTECT_SECTOR;  next_token = 2; }
	                  else if (!strcmp( Tokens[1], ".UC"))  {arg0 = UNPROTECT_CHIP;  next_token = 2; }
	                  else
	                     {
	                     UART_printf( "\r\nFLASH parms wrong\r\n");
	                     } // end else malformed command
	
	
	                   //sprintf(sbuffer, "\nparsed: CMD = %d", arg0);                    
	                   //UART_printf( sbuffer );
	
	                   // what is the command?
	                   switch( arg0 )
	                   {
	                   case UNPROTECT_CHIP:
	                        {
	
	                        // unprotect the chip sector by sector, default on power up is protected, pain, but we have to do this...
	
	                        UART_printf("\r\nExec Unprotect chip...");
	
	                        // step 1: send WRITE_ENABLE command
	
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	                        spi_data = SPI_WriteRead( WRITE_ENABLE );
	
	                        // disable SPI interface
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);
	
	                        // step 2: unprotect all sectors from sector address list
	                        for (index = 0; index < num_flash_sectors; index++)
	                            {
	                            // enable SPI interface
	                            // set CS to SPI select channel 3 (FLASH)
	                            SPI_SET_CS(SPI_CS_FLASH);
	
	                            spi_data = SPI_WriteRead( UNPROTECT_SECTOR );
	
	                            // write sector address, there are only a handful of sectors since each is 4-64K
	                            spi_data = SPI_WriteRead( sector_addresses[ index ] >> 16 );
	                            spi_data = SPI_WriteRead( sector_addresses[ index ] >> 8 );
	                            spi_data = SPI_WriteRead( sector_addresses[ index ] >> 0 );
	
	                            // disable SPI interface
	                            // set CS to SPI select channel 0 (null)
	                            SPI_SET_CS(SPI_CS_0);
	
	                            // progress bar..    
	                            UART_putchar('.');
	
	                            DELAY_MS(1000);
	
	                            } // end for index
	
	                        } break;
	
	
	                    case UNPROTECT_SECTOR:
	                        {
	                        // get address for sector to unprotect
	                        if ( (num_Tokens-1) >= next_token )
	                          arg1 = _atol2( Tokens[ next_token ] );
	                        else
	                          arg1 = 0; // default to sector 0
	
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	
	                        UART_printf("\r\nExec Unprotect Sec Cmd...\r\n");
	                        spi_data = SPI_WriteRead( UNPROTECT_SECTOR );

							flash_start_addr = arg1;
	
	                        // write sector address, there are only a handful of sectors since each is 4-64K
	                        spi_data = SPI_WriteRead( flash_start_addr >> 16 );
	                        spi_data = SPI_WriteRead( flash_start_addr >> 8 );
	                        spi_data = SPI_WriteRead( flash_start_addr >> 0 );
	
	                        // disable SPI interface
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);
	
	                        } break;
	
	                    case WRITE_ENABLE:
	                        {
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	
	                        UART_printf("\r\nExec Write Enable...\r\n");
	                        spi_data = SPI_WriteRead( WRITE_ENABLE );
	
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);
	
	                        } break;

	                    case WRITE_DISABLE: 
	                        {
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	
	                        UART_printf("\r\nExec Write Disable...\r\n");
	                        spi_data = SPI_WriteRead( WRITE_DISABLE );
	
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);
	                        } break;
	
	                    case READ_STATUS:     
	                        {
	                        // check for iterations command, any Tokens left?
	                        if ( (num_Tokens-1) >= next_token )
	                          arg1 = _atol2( Tokens[ next_token ] );
	                        else
	                          arg1 = 1;
	
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	
	                        UART_printf("\r\nExec Read Stat Reg...");
	                        spi_data = SPI_WriteRead( READ_STATUS );
	                  
	                        for (index = 0; index < arg1; index++)
	                            {
	                            spi_data = SPI_WriteRead( 0xFF );
	
	                            sprintf(sbuffer, "\r\nStatus = %d\r\n", (int)spi_data);                    
	                            UART_printf( sbuffer );
	
	                            } // end for
	
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);
	
	                        } break;

	                    case WRITE_STATUS:   
	                        {
	
	                        // check for iterations command, any Tokens left?
	                        if ( (num_Tokens-1) >= next_token )
	                          arg1 = _atol2( Tokens[ next_token ] );
	                        else
	                          arg1 = 0;
	
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	
	                        UART_printf("\r\nExec Write Status Reg...\r\n");
	                        spi_data = SPI_WriteRead( WRITE_STATUS );
	                  
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);

	                        } break;

	                    case JEDEC_ID:
	                        {
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	
	                        UART_printf("\r\nExec JEDEC ID...");
	
	                        spi_data = SPI_WriteRead( JEDEC_ID );
	                    
	                        // now read the id data back
	                        
	                        sprintf(sbuffer,"\r\nManf = %d,", SPI_WriteRead( 0x00 ) );
	                        UART_printf( sbuffer );                            
	                                                      
	                        sprintf(sbuffer,"\r\nMem Type = %d,", SPI_WriteRead( 0x00 ) );
	                        UART_printf( sbuffer );                            
	
	                        sprintf(sbuffer,"\r\nCap = %d\r\n", SPI_WriteRead( 0x00 ) );
	                        UART_printf( sbuffer );                            
	
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);
	
	                        } break;
	
	                    case READ_DATA:       
	                        {
	                         // set CS to SPI select channel 3 (FLASH)
	                         SPI_SET_CS(SPI_CS_FLASH);
	
	                         UART_printf("\r\nExec Read Byte(s)...");
	
	                         flash_start_addr = _atol2( Tokens[ 2 ] );
	                         flash_count      = _atol2( Tokens[ 3 ] ); 
	                     
	                         // initiate read command
	                         spi_data = SPI_WriteRead( READ_DATA );
	
	                         // write starting address byte by byte, MSB first...
	                         spi_data = SPI_WriteRead( flash_start_addr >> 16 );
	                         spi_data = SPI_WriteRead( flash_start_addr >> 8 );
	                         spi_data = SPI_WriteRead( flash_start_addr >> 0 );
	
	                         // now read data 
	                         for (index = 0; index < flash_count; index++)
	                             {
	                             flash_value = SPI_WriteRead( 0x00 );
	
	                             sprintf(sbuffer,"%ld,",flash_value);
	                             UART_printf( sbuffer );                            
	
	                             } // end for index

							UART_printf("\r\n");	

	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);
	                        
	                        } break;

	                    case SEQ_PROGRAM: 
	                        {
	
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	
	                         UART_printf("\r\nExec Seq Pgm...");
	
	                         flash_start_addr = _atol2( Tokens[ 2 ] );
	                         flash_count      = _atol2( Tokens[ 3 ] ); 
	                         flash_value      = _atol2( Tokens[ 4 ] ); 
	
	
	                         sprintf(sbuffer,"\r\n%ld %ld %ld",flash_start_addr,
	                                                       flash_count,
	                                                       flash_value);
	
	
	                         UART_printf( sbuffer );                            
	
	                         // initiate seq program command
	                         spi_data = SPI_WriteRead( SEQ_PROGRAM );
	
	                         // write starting address byte by byte, MSB first...
	                         spi_data = SPI_WriteRead( flash_start_addr >> 16 );
	                         spi_data = SPI_WriteRead( flash_start_addr >> 8 );
	                         spi_data = SPI_WriteRead( flash_start_addr >> 0 );
	
	                         sprintf(sbuffer,"\r\nWrt %ld from [%ld..%ld]",flash_value, 
	                                                                     flash_start_addr, 
	                                                                     flash_start_addr + flash_count-1);
	                                                                     
	                         UART_printf( sbuffer );                            
	
	                         // write first byte
	                         spi_data = SPI_WriteRead( flash_value );
	
	                         // de-assert set CS to SPI select channel 0 (null)
	                         SPI_SET_CS(SPI_CS_0);
	
	                         
	                         // now write remaining data
	                         for (index = 0; index < flash_count; index++)
	                             {
	
	                             // set CS to SPI select channel 3 (FLASH)
	                             SPI_SET_CS(SPI_CS_FLASH);
	                  
	                             // re-issue seq program command, but NO address needed
	                             spi_data = SPI_WriteRead( SEQ_PROGRAM );
	                             // now data
	                             spi_data = SPI_WriteRead( flash_value );
	
	                             // set CS to SPI select channel 0 (null)
	                             SPI_SET_CS(SPI_CS_0);
	
	                             // progress bar..    
	                             UART_putchar('.');
	
	                             } // end for index
	
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);
	
	                        // finally send write disable command
	
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	                        
	                        spi_data = SPI_WriteRead( WRITE_DISABLE );
	                        
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);

							UART_printf("\r\n");
		
	                        } break;

	                    case BYTE_PROGRAM: 
	                        {
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	
	                         UART_printf("\r\nExec byte Pgm...");
	
	                         flash_start_addr = _atol2( Tokens[ 2 ] );
	                         flash_count      = _atol2( Tokens[ 3 ] ); 
	                         flash_value      = _atol2( Tokens[ 4 ] ); 
	
	
	                         sprintf(sbuffer,"\r\n%ld %ld %ld",flash_start_addr,
	                                                       flash_count,
	                                                       flash_value);
	
	
	                         UART_printf( sbuffer );                            
	
	                         // initiate byte program command
	                         spi_data = SPI_WriteRead( BYTE_PROGRAM );
	
	                         // write starting address byte by byte, MSB first...
	                         spi_data = SPI_WriteRead( flash_start_addr >> 16 );
	                         spi_data = SPI_WriteRead( flash_start_addr >> 8 );
	                         spi_data = SPI_WriteRead( flash_start_addr >> 0 );
	
	                         sprintf(sbuffer,"\r\nWrt %ld from [%ld..%ld]\r\n",flash_value, 
	                                                                     flash_start_addr, 
	                                                                     flash_start_addr + flash_count-1);
	                                                                     
	                         UART_printf( sbuffer );                            
	
	                         // write  byte
	                         spi_data = SPI_WriteRead( flash_value );
	
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);
	
	                        } break;
	
	                    case BLOCK_ERASE4:     
	                        {
	                        // get address for block to erase
	                        if ( (num_Tokens-1) >= next_token )
	                          flash_start_addr = _atol2( Tokens[ next_token ] );
	                        else
	                          flash_start_addr = 0; // default to block 0
	
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	
	                        UART_printf("\r\nExec block erase 4K Cmd...\r\n");
	                        spi_data = SPI_WriteRead( BLOCK_ERASE4 );
	
	                        // write block address, there are only a handful of blocks since each is 4-64K
	                        spi_data = SPI_WriteRead( flash_start_addr >> 16 );
	                        spi_data = SPI_WriteRead( flash_start_addr >> 8 );
	                        spi_data = SPI_WriteRead( flash_start_addr >> 0 );
	
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);
	
	                        } break;
	

	                    case CHIP_ERASE:    
	                        {
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	
	                        UART_printf("\r\nExec Chip Erase...");
	                        spi_data = SPI_WriteRead( CHIP_ERASE );
	
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);
	
	                        // set CS to SPI select channel 3 (FLASH)
	                        SPI_SET_CS(SPI_CS_FLASH);
	
	                        // poll spi interface, wait for busy to be 0
	                        spi_data = SPI_WriteRead( READ_STATUS );
	
	                        // progress bar counter
	                        index=0; 
	                
	                        do {
	
	                           // progress bar..    
	                           UART_putchar('.');
	
	                           DELAY_MS(1000);
	
	                           } while (SPI_WriteRead( 0xFF ) & 0x01 );
	
	
	                        // set CS to SPI select channel 0 (null)
	                        SPI_SET_CS(SPI_CS_0);

							UART_printf("\r\n");
	
	                        } break;

	                    default: 
	                        {
	                        UART_printf( "\r\nCmd not supported.\r\n");
	                        } break;
	                    } // end switch
	
                    } // end if FLASH
					else
					{
						// If we got here we have an invalid command
						if(NumTokens)
							UART_printf("\r\nInvalid Command (%s)\r\n",Tokens[0]);
						else
							UART_printf("\r\n");
					}

					// Display our command line prompt
					UART_printf("READY>");

					// Finished with this command....get a new one.
					CmdStrAddPtr = 0;

				} // end if carriage return
			} // end if other character
		} // end if received a character
	} // end while(1)
}



void _strlwr(char *CmdStr)
{
	if(CmdStr == NULL) return;
	while(*CmdStr)
	{
		if(IsAlphaUpr(*CmdStr))
			*CmdStr += 32;
		CmdStr++;
	}
}

void _strupr(char *CmdStr)
{
	if(CmdStr == NULL) return;
	while(*CmdStr)
	{
		if(IsAlphaLwr(*CmdStr))
			*CmdStr -= 32;
		CmdStr++;
	}
}


long _atol2(char *Str)
{
	long RetVal = 0;

	if(Str == NULL) return 0;

	switch(Str[0])
	{
	case '$':
		// Hex
		Str++;
		while(*Str != 0)
		{
			RetVal = RetVal << 4;
			if(*Str >= 48 && *Str <= 57)
				RetVal |= *Str - 48;
			else if(*Str >= 65 && *Str <= 70)
				RetVal |= *Str - 65 + 10;
			else
				return 0; // Invalid character
			Str++;
		}

		return RetVal;
	break;
	case '%':
		// Binary
		Str++;
		while(*Str != 0)
		{
			RetVal = RetVal << 1;
			if(*Str == '1')
				RetVal |= 0x01;
			else if(*Str != '0')
				return 0;
			Str++;
		}

		return RetVal;
	break;
	default:
		// Normal atol
		return atol(Str);
	break;
	}
}


int atoi2(char *Str)
{
	int RetVal = 0;

	if(Str == NULL) return 0;

	switch(Str[0])
	{
	case '$':
		// Hex
		Str++;
		while(*Str != 0)
		{
			RetVal = RetVal << 4;
			if(*Str >= 48 && *Str <= 57)
				RetVal |= *Str - 48;
			else if(*Str >= 65 && *Str <= 70)
				RetVal |= *Str - 65 + 10;
			else
				return 0; // Invalid character
			Str++;
		}

		return RetVal;
	break;
	case '%':
		// Binary
		Str++;
		while(*Str != 0)
		{
			RetVal = RetVal << 1;
			if(*Str == '1')
				RetVal |= 0x01;
			else if(*Str != '0')
				return 0;
			Str++;
		}

		return RetVal;
	break;
	default:
		// Normal atol
		return atoi(Str);
	break;
	}
}

