///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright Nurve Networks LLC 2009
//
// Filename: CHAM_PIC_GFX_DRV_V010.C
//
// Original Author: Andre' LaMothe
//                  Joshua Hintze
//
// Last Modified: 8.28.09
//
// Description: This is GFX interface API for the Default2 driver which supports the HEL tile engine with color tiles,
// scrolling, large playfields, overscan colors etc.
//
// Overview:
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


// include local Chameleon PIC API header files
#include "CHAM_PIC_SYSTEM_V010.h"
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"
#include "CHAM_PIC_GFX_DRV_V010.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS /////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TYPES/CLASSES //////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS ////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS //////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS  /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Graphics API for default driver 2 (NTSC terminal commands still work)
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Set_Top_Overscan_Color(int col)
{
// sets the 8-bit top overscan color of the tile mode
// GPU_GFX_TOP_OVERSCAN_COL_W = (13+GPU_GFX_BASE_ID)
SPI_Prop_Send_Cmd( GPU_GFX_TOP_OVERSCAN_COL_W , col, 0x00 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

}  // GFX_Set_Top_Overscan_Color

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Set_Bottom_Overscan_Color(int col)
{
// sets the 8-bit bottom overscan color of the tile mode
SPI_Prop_Send_Cmd( GPU_GFX_BOT_OVERSCAN_COL_W, col, 0x00 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_Set_Bottom_Overscan_Color

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Set_Tilemap_Width(int width)
{
// sets the width of the tile map, 32, 64, 128, 256 are available and encoded as 0,1,2,3
SPI_Prop_Send_Cmd( GPU_GFX_SCREEN_WIDTH_W, width, 0x00 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_Set_Tilemap_Width

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Get_Top_Overscan_Color(void)
{
// retrieves the 8-bit top overscan color
int result;

// send read command: cmd, 0,0
SPI_Prop_Send_Cmd( GPU_GFX_TOP_OVERSCAN_COL_R, 0x00, 0x00 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now retrieve results, note, if we read the results back too soon, the driver might not have time to
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end GFX_Get_Top_Overscan_Color

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Get_Bottom_Overscan_Color(void)
{
// retrieves the 8-bit bottom overscan color

int result;

// send read command: cmd, 0,0
SPI_Prop_Send_Cmd( GPU_GFX_BOT_OVERSCAN_COL_R, 0x00, 0x00);

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now retrieve results, note, if we read the results back too soon, the driver might not have time to
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end GFX_Get_Bottom_Overscan_Color

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Get_Tilemap_Width(void)
{
// retrieve 8-bit the tile map width encoded 0,1,2,3 = (32, 64, 128, 256)
int result;

// send read command: cmd, 0,0
SPI_Prop_Send_Cmd( GPU_GFX_SCREEN_WIDTH_W, 0x00, 0x00);

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now retrieve results, note, if we read the results back too soon, the driver might not have time to
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end GFX_Get_Tilemap_Width

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Set_Dest_Ptr(unsigned int dest_ptr)
{
// sets the 16-bit destination pointer in the tile GPU for memory transfer and fill operations
SPI_Prop_Send_Cmd( GPU_GFX_DEST_ADDR_W, dest_ptr & 0xFF, dest_ptr >> 8 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_Set_Dest_Ptr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Set_Src_Ptr(unsigned int src_ptr)
{
// sets the 16-bit source pointer in the tile GPU for memory transfer and fill operations
SPI_Prop_Send_Cmd( GPU_GFX_SRC_ADDR_W, src_ptr & 0xFF, src_ptr >> 8 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_Set_Src_Ptr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Get_Dest_Ptr(void)
{
// retrieves the 16-bit destination pointer in the tile GPU for memory transfer and fill operations
int result;

// send read command: cmd, 0,0
SPI_Prop_Send_Cmd( GPU_GFX_DEST_ADDR_R, 0x00, 0x00);

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now retrieve results, note, if we read the results back too soon, the driver might not have time to
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end GFX_Get_Dest_Ptr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Get_Src_Ptr(void)
{
// retrieves the 16-bit source pointer in the tile GPU for memory transfer and fill operations
int result;

// send read command: cmd, 0,0
SPI_Prop_Send_Cmd( GPU_GFX_SRC_ADDR_R, 0x00, 0x00);

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now retrieve results, note, if we read the results back too soon, the driver might not have time to
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end GFX_Get_Src_Ptr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Write_Mem_Port16(unsigned int data)
{
// writes a 16-bit value to prop_mem[ source_ptr ] , note source pointer is always used for memory port operations
SPI_Prop_Send_Cmd( GPU_GFX_RAM_PORT16_W, data & 0xFF, data >> 8 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_Write_Mem_Port16

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Read_Mem_Port16(void)
{
// reads a 16-bit value from prop_mem[ source_ptr ] , note source pointer is always used for memory port operations

int result;

// send read command: cmd, 0,0
SPI_Prop_Send_Cmd( GPU_GFX_RAM_PORT16_R, 0x00, 0x00);

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now retrieve results, note, if we read the results back too soon, the driver might not have time to
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end GFX_Read_Mem_Port16

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Write_Mem_Port8(unsigned int data)
{
// writes a 8-bit value to prop_mem[ source_ptr ] , note source pointer is always used for memory port operations
SPI_Prop_Send_Cmd( GPU_GFX_RAM_PORT8_W, data & 0xFF, 0x00 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_Write_Mem_Port8

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Read_Mem_Port8(void)
{
// reads a 8-bit value to prop_mem[ source_ptr ] , note source pointer is always used for memory port operations

int result;

// send read command: cmd, 0,0
SPI_Prop_Send_Cmd( GPU_GFX_RAM_PORT8_R, 0x00, 0x00);

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now retrieve results, note, if we read the results back too soon, the driver might not have time to
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end GFX_Read_Mem_Port18

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Set_VScroll(int vscroll)
{
// sets the 8-bit vertical scroll register 0..7
SPI_Prop_Send_Cmd( GPU_GFX_VSCROLL_FINE_W, vscroll & 0x07, 0x00 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // GFX_Set_VScroll

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Set_HScroll(void)
{
// not implemented in driver yet
// return success
return(1);

} // end GFX_Set_HScroll

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Get_VScroll(void)
{
// retrieves the 8-bit current vertical scroll 0..7

int result;

// send read command: cmd, 0,0
SPI_Prop_Send_Cmd( GPU_GFX_VSCROLL_FINE_R, 0x00, 0x00);

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now retrieve results, note, if we read the results back too soon, the driver might not have time to
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // GFX_Get_VScroll

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_Get_HScroll(void)
{
// not implemented in driver yet

// return success
return(1);

} // end GFX_Get_HScroll

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Write_Bitmap_Ptr(unsigned int bitmap_ptr)
{
// writes the 16-bit bitmap pointer that points to the first bitmap indexed by tile 0. Each bitmap is 8x8 pixels, 2-bits
// per pixel, 16-bytes each.
SPI_Prop_Send_Cmd( GPU_GFX_BITMAP_W, bitmap_ptr & 0xFF, bitmap_ptr >> 8 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_Write_Bitmap_Ptr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Read_Bitmap_Ptr(void)
{
// reades the 16-bit bitmap pointer that points to the first bitmap indexed by tile 0. Each bitmap is 8x8 pixels, 2-bits
// per pixel, 16-bytes each.

int result;

// send read command: cmd, 0,0
SPI_Prop_Send_Cmd( GPU_GFX_BITMAP_R, 0x00, 0x00);

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now retrieve results, note, if we read the results back too soon, the driver might not have time to
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end GFX_Read_Bitmap_Ptr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Write_Tilemap_Ptr(unsigned int tilemap_ptr)
{
// writes the 16-bit tilemap pointer that points to the current tile map data displayed. Each tile map entry is 2 bytes
// [palette index | character index], and the current tile map width decides the pitch

SPI_Prop_Send_Cmd( GPU_GFX_TILE_MAP_W, tilemap_ptr & 0xFF, tilemap_ptr >> 8 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_Write_Tilemap_Ptr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Read_Tilemap_Ptr(void)
{
// reades the 16-bit tilemap pointer that points to the current tile map data displayed. Each tile map entry is 2 bytes
// [palette index | character index], and the current tile map width decides the pitch

int result;

// send read command: cmd, 0,0
SPI_Prop_Send_Cmd( GPU_GFX_TILE_MAP_R, 0x00, 0x00);

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now retrieve results, note, if we read the results back too soon, the driver might not have time to
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end GFX_Read_Tilemap_Ptr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Write_Palette_Ptr(unsigned int palette_ptr)
{
// writes the 16-bit palette pointer that points to the set of palettes referred to by the palette index in each tile entry
// each palette entry is a single 32-bit long that represents 4 colors [color 3 | color 2 | color 1 | color 0] that are represented
// by the bitmaps as each 2-bit pair indexing as color 0,1,2,3.

SPI_Prop_Send_Cmd( GPU_GFX_PALETTE_W, palette_ptr & 0xFF, palette_ptr >> 8 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_Write_Palette_Ptr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

unsigned int GFX_Read_Palette_Ptr(void)
{
// reades the 16-bit palette pointer that points to the set of palettes referred to by the palette index in each tile entry
// each palette entry is a single 32-bit long that represents 4 colors [color 3 | color 2 | color 1 | color 0] that are represented
// by the bitmaps as each 2-bit pair indexing as color 0,1,2,3.

int result;

// send read command: cmd, 0,0
SPI_Prop_Send_Cmd( GPU_GFX_PALETTE_R, 0x00, 0x00);

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now retrieve results, note, if we read the results back too soon, the driver might not have time to
// set them, so keep that in mind
result = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00);

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

return ( result );

} // end GFX_Read_Palette_Ptr

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GPU commands, executed locally on driver COG, faster
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_GPU_Copy_Mem8(unsigned int dest_ptr, unsigned int src_ptr, unsigned int num_bytes)
{
// this command copies a number of bytes from source to destination in the GPU memory space, or in other words in the Prop's
// local memory of 32K, you can use this to overwrite SPIN code, so watch out!

// set the destination pointer
SPI_Prop_Send_Cmd( GPU_GFX_DEST_ADDR_W, dest_ptr & 0xFF, dest_ptr >> 8 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// set the source pointer
SPI_Prop_Send_Cmd( GPU_GFX_SRC_ADDR_W, src_ptr & 0xFF, src_ptr >> 8 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// set the number of bytes to copy
SPI_Prop_Send_Cmd( GPU_GFX_NUM_BYTES_W, num_bytes & 0xFF, num_bytes >> 8 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// execute the GPU commmand
SPI_Prop_Send_Cmd( GPU_GFX_SUBFUNC_STATUS_W, GPU_GFX_SUBFUNC_COPYMEM8, 0x00 );

// let prop finish transaction
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_GPU_Copy_Mem8

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_GPU_Fill_Mem8(unsigned int dest_ptr,unsigned int value, unsigned int num_bytes)
{
// this command fills a number of bytes at the destination in the GPU memory space with a specific value, or in other words
// in the Prop's local memory of 32K, you can use this to overwrite SPIN code, so watch out!

// set the destination pointer
SPI_Prop_Send_Cmd( GPU_GFX_DEST_ADDR_W, dest_ptr & 0xFF, dest_ptr >> 8 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// set the data to fill with
SPI_Prop_Send_Cmd( GPU_GFX_DATA_W, value & 0xFF, 0x00 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// set the number of bytes to fill
SPI_Prop_Send_Cmd( GPU_GFX_NUM_BYTES_W, num_bytes & 0xFF, num_bytes >> 8 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// execute the GPU commmand
SPI_Prop_Send_Cmd( GPU_GFX_SUBFUNC_STATUS_W, GPU_GFX_SUBFUNC_FILLMEM8, 0x00 );

// let prop get data ready, this could take a while, so some intelligence here is in order.
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_GPU_Fill_Mem8

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_GPU_Copy_Mem16(unsigned int dest_ptr, unsigned int src_ptr, unsigned int num_bytes)
{
// this command copies a number of bytes from source to destination in the GPU memory space in WORD side chunks, or in other
// words in the Prop's local memory of 32K, you can use this to overwrite SPIN code, so watch out!

// set the destination pointer
SPI_Prop_Send_Cmd( GPU_GFX_DEST_ADDR_W, dest_ptr & 0xFF, dest_ptr >> 8 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// set the source pointer
SPI_Prop_Send_Cmd( GPU_GFX_SRC_ADDR_W, src_ptr & 0xFF, src_ptr >> 8 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// set the number of bytes to copy
SPI_Prop_Send_Cmd( GPU_GFX_NUM_BYTES_W, num_bytes & 0xFF, num_bytes >> 8 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// execute the GPU commmand
SPI_Prop_Send_Cmd( GPU_GFX_SUBFUNC_STATUS_W, GPU_GFX_SUBFUNC_COPYMEM16, 0x00 );

// let prop get data ready, this could take a while, so some intelligence here is in order.
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_GPU_Copy_Mem16

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int GFX_GPU_Fill_Mem16(unsigned int dest_ptr,unsigned int value, unsigned int num_bytes)
{
// this command fills a number of bytes at the destination in the GPU memory space with a specific WORD sized value,
// or in other words in the Prop's local memory of 32K, you can use this to overwrite SPIN code, so watch out!

// set the destination pointer
SPI_Prop_Send_Cmd( GPU_GFX_DEST_ADDR_W, dest_ptr & 0xFF, dest_ptr >> 8 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// set the data to fill with
SPI_Prop_Send_Cmd( GPU_GFX_DATA_W, value & 0xFF, value >> 8 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// set the number of bytes to fill
SPI_Prop_Send_Cmd( GPU_GFX_NUM_BYTES_W, num_bytes & 0xFF, num_bytes >> 8 );

// let prop get data ready
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// execute the GPU commmand
SPI_Prop_Send_Cmd( GPU_GFX_SUBFUNC_STATUS_W, GPU_GFX_SUBFUNC_FILLMEM16, 0x00 );

// let prop get data ready, this could take a while, so some intelligence here is in order.
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end GFX_GPU_Fill_Mem16

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

