///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright Nurve Networks LLC 2008
//
// Filename: CHAM_PIC_HELLO_WORLD_01.C
//
// Original Author: Joshua Hintze
//
// Last Modified: 10.14.09
//
// Description: Simple Hello World Application that outputs text on VGA and NTSC.
//
//
// Required Files: CHAM_PIC_SYSTEM_V010.c/h
//				   CHAM_PIC_I2C_SPI_DRV_V010.c/h
//				   CHAM_PIC_NTSC_DRV_V010.c/h
//				   CHAM_PIC_VGA_DRV_V010.c/h
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>


// Include main system header
#include "CHAM_PIC_SYSTEM_V010.h"

// Include spi driver
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"

// Include the VGA/NTSC displays
#include "CHAM_PIC_NTSC_DRV_V010.h"
#include "CHAM_PIC_VGA_DRV_V010.h"

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Our main start location
int main (void)
{
	// Always call ConfigureClock first
	SYS_ConfigureClock(MAX_FCY_RATE);

	// Initialize the spi interface
	SPI_SET_CS(SPI_CS_0);
	SPI_Init();

	// Wait for the propeller to initialize
	DELAY_SEC(3);

	// Clear screen
	NTSC_ClearScreen();
	VGA_ClearScreen();

	// enter infinite loop...
	while(1)
     {
		 // print on NTSC terminal screen
		 NTSC_Color(0);
		 NTSC_Term_Print("Hello World!   ");

		 // print on VGA terminal screen
		 VGA_Color(0);
		 VGA_Term_Print("Hello World!   ");

		 // slow things down a bit, so we can read the text!
		 DELAY_MS(10);

     } // end while

}

