///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright Nurve Networks LLC 2009
//
// Filename: CHAM_PIC_KEYBOARD_DEMO_01.C
//
// Original Author: Andre' LaMothe
//
// Last Modified: 10.18.09
//
// Description: Read local PS/2 keyboard connected to Chameleon prints the keys as user presses them.
//
// Overview: Prints local PS/2 keyboard key presses to both NTSC and VGA screens. Make sure to have keyboard plugged in
//           and hit reset to start program.
//
// Chameleon AVR version nearly identical
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// include headers required for demo
#include "CHAM_PIC_SYSTEM_V010.h"             // you need this one always
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"        // you need this always
#include "CHAM_PIC_NTSC_DRV_V010.h"           // needed for NTSC driver
#include "CHAM_PIC_VGA_DRV_V010.h"            // needed for VGA driver
#include "CHAM_PIC_KEYBOARD_DRV_V010.h"       // needed for keyboard driver

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES AND MACROS /////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <string.h>
#include <stdio.h>

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS ////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// a little fun
char access_code[] = "morpheous";
int access_code_index = 0;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MAIN ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(void)
{
int key;          // key that keyboard returns
char sbuffer[64]; // printing buffer

// Always call ConfigureClock first
SYS_ConfigureClock(MAX_FCY_RATE);

// initialize SPI
SPI_Init();

// give Propeller driver time to boot
DELAY_MS( 2500 );

// clear screens
NTSC_ClearScreen();
VGA_ClearScreen();

// set NTSC color
NTSC_Color(0);
NTSC_Term_Print(" PS/2 Keyboard Demo");
NTSC_Term_Char( 0x0D );
NTSC_Term_Print(" Press keys on local keyboard.");
NTSC_Term_Char( 0x0D );

// set VGA color
VGA_Color(0);
VGA_Term_Char( 0x0D );
VGA_Term_Print(" Press keys on local keyboard.");
VGA_Term_Char( 0x0D );

// load keyboard driver (this driver is already loaded by default, but this call can't hurt)
Keyboard_Load();

// enter infinite loop...
while(1)
     {
     // read next key, return 0 (null if no key), wait for key press
     while ( (key = Keyboard_Key()) == 0);

     // build display string
     if (key >= 32)
        sprintf(sbuffer, "Key Code = %d [%c]                ", (int)key, (char)key);
     else
        sprintf(sbuffer, "Key Code = %d [non-printable]     ", (int)key);

     // position cursor
     NTSC_SetXY(1, 4);

     // print on NTSC terminal screen
     NTSC_Term_Print(sbuffer);
     NTSC_Term_Char( 0x0D );

     // position cursor
     VGA_SetXY(1, 4);

     // print on VGA terminal screen
     VGA_Term_Print(sbuffer);
     VGA_Term_Char( 0x0D );

     //////////////////////////////////////////////////////////////////////////////////
     // little string compare state machine for fun, when user types in access code
     // it displays a little message at bottom
     if (key == access_code[ access_code_index ])
        {
        // consume character, test for match?
        if (++access_code_index == strlen(access_code))
           {
           NTSC_SetXY(1, 22);
           NTSC_Term_Print("Welcome NEO...Access Granted.");

           VGA_SetXY(1, 14);
           VGA_Term_Print("Welcome NEO...Access Granted.");

           access_code_index = 0;

           } // end if match

        } // end end if
     else // reset access code, start over
       access_code_index = 0;
     //////////////////////////////////////////////////////////////////////////////////

     // slow things down a bit, so we can read the text!
     DELAY_MS(10);

     } // end while

} // end main
