///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright Nurve Networks LLC 2009
//
// Filename: CHAM_PIC_NTSC_GLOW_DEMO_01.c
//
// Original Author: Andre' LaMothe
//
// Last Modified: 10.18.09
//
// Description: NTSC demo that "glows" the top and bottom overscan region using some of the NTSC specific
// functionality in the Default2 version of the Propeller driver.
//
// Overview: Simply prints out some text then glows the top and bottom overscan, by slowing cycling thru all the colors
// in the palette.
//
// Chameleon AVR version nearly identical
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



// include headers required for demo
#include "CHAM_PIC_SYSTEM_V010.h"             // you need this one always
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"        // you need this always
#include "CHAM_PIC_NTSC_DRV_V010.h"           // needed for NTSC driver
#include "CHAM_PIC_GFX_DRV_V010.h"			  // needed for gfx command

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES AND MACROS /////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MAIN ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(void)
{
unsigned char color, bright, col;

// Always call ConfigureClock first
SYS_ConfigureClock(MAX_FCY_RATE);

// initialize SPI
SPI_Init();


// give Prop a moment before sending it commands, the boot process is about a second
// if you like you can speed it up, by removing the Prop driver's "LED blink" sequence in the driver
// or speed it up, but commands that are sent before the Prop driver is done booting will be ignored.
// therefore, if you have a lot of set up work to do, then you don't require a delay, but if you jump right
// into commands, then you need a good 1.5 - 2 second delay
DELAY_MS( 2500 );

// clear screens
NTSC_ClearScreen();

// print on NTSC terminal screen
NTSC_Color(0);

// print text
NTSC_Term_Print("Cylone Base Star to Outpost...");
NTSC_Term_Char( 0x0D ); // newline
NTSC_Term_Char( 0x0D ); // newline
DELAY_MS( 1000 );

NTSC_Term_Print("Message is:");
NTSC_Term_Char( 0x0D ); // newline
NTSC_Term_Char( 0x0D ); // newline
DELAY_MS( 2000 );

NTSC_Term_Print("Destroy Caprica!");
NTSC_Term_Char( 0x0D ); // newline
DELAY_MS( 500 );

NTSC_Term_Print("Destroy Caprica!");
NTSC_Term_Char( 0x0D ); // newline
DELAY_MS( 500 );

NTSC_Term_Print("Destroy Caprica!");
NTSC_Term_Char( 0x0D ); // newline
DELAY_MS( 1500 );

// position the cursor and print in center
NTSC_SetXY(8, 12);
NTSC_Term_Print("BY YOUR COMMAND...");

// enter infinite loop...glowing the top and bottom overscan, tile engine functions required for this
while(1)
     {
     // glow thru every color, every brightness
     for (col = 0; col <= 15 ; col++)
         {
         for (bright = 2; bright < 6; bright++)
            {

            // build up Propeller compatible color [chroma:4 | chroma bit:1 | luma:3]
            color = (col << 4) | (0b00001000) | (bright);

            // set top overscan
            GFX_Set_Top_Overscan_Color( color & 0xFF );

            // set bottom overscan
            GFX_Set_Bottom_Overscan_Color( color  & 0xFF );

            // slow things down a bit, so we can read the text!
            DELAY_MS(100);

            } // end for bright

       } // end for color

     } // end while

} // end main
