///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright Nurve Networks LLC 2009
//
// Filename: CHAM_PIC_NTSC_SMOOTH_SCROLL_DEMO_01.c
//
// Original Author: Andre' LaMothe
//
// Last Modified: 10.18.09
//
// Description: NTSC demo of smooth scrolling with the advaced NTSC tile engine.
//
// Overview: Simply smooth scrolls some tiles on the screen up and down with the fine scroll register.
// Chameleon AVR version nearly identical
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


// include headers required for demo
#include "CHAM_PIC_SYSTEM_V010.h"             // you need this one always
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"        // you need this always
#include "CHAM_PIC_NTSC_DRV_V010.h"           // needed for NTSC driver
#include "CHAM_PIC_GFX_DRV_V010.h"            // needed for special GFX commands like fine scrolling

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES AND MACROS /////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


#define SPACE_INVADERS_CHAR_OFFSET (32+4*32+13) // 32 is where "blank/space" is mapped, so our character set starts from 32...
#define SPACE_INVADERS_ORIGIN_X    6
#define SPACE_INVADERS_ORIGIN_Y    8
#define NUM_SPACE_INVADERS_PER_ROW 10

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MAIN ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(void)
{
int x,y, scroll_y = 0, scroll_dy = 1;

// Always call ConfigureClock first
SYS_ConfigureClock(MAX_FCY_RATE);

// initialize SPI
SPI_Init();


// give Prop a moment before sending it commands, the boot process is about a second
// if you like you can speed it up, by removing the Prop driver's "LED blink" sequence in the driver
// or speed it up, but commands that are sent before the Prop driver is done booting will be ignored.
// therefore, if you have a lot of set up work to do, then you don't require a delay, but if you jump right
// into commands, then you need a good 1.5 - 2 second delay
DELAY_MS( 2500 );

// clear screens
NTSC_ClearScreen();

// basic white
NTSC_Color(0);

// draw a matrix of space invaders with extended characters
for (y = 0; y < 6; y++)
    {
    for (x = 0; x < NUM_SPACE_INVADERS_PER_ROW; x++)
        {
        // position next invader
        NTSC_SetXY(x*2 + SPACE_INVADERS_ORIGIN_X , y*2 + SPACE_INVADERS_ORIGIN_Y);

        // draw next invader
        NTSC_Term_Char( SPACE_INVADERS_CHAR_OFFSET + (y % 3) );
        } // end for x
    } // end for y

// enter infinite loop...
while(1)
     {
     // set smooth scroll vertical position register 0..7
     GFX_Set_VScroll( scroll_y );

     // change scroll position
     scroll_y += scroll_dy;

     // test for limit
     if ( (scroll_y > 7) || (scroll_y < 0) )
        {
        scroll_dy = -scroll_dy;
        scroll_y += scroll_dy;
        } // end if

     // slow things down a bit, so we can read the text!
     DELAY_MS(50);

     } // end while

} // end main
