///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright Nurve Networks LLC 2009
//
// Filename: CHAM_PIC_SOUND_DEMO_01.C
//
// Original Author: Andre' LaMothe
//
// Last Modified: 10.18.09
//
// Description: Plays a little song using the sound driver.
//
// Overview: Plays a little song on the sound driver, outputs notes on both NTSC and VGA terminals.
// Chameleon AVR version nearly identical
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// Include standard C headers
#include <stdio.h>
#include <math.h>

// include headers required for demo
#include "CHAM_PIC_SYSTEM_V010.h"             // you need this one always
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"        // you need this always
#include "CHAM_PIC_NTSC_DRV_V010.h"           // needed for NTSC driver
#include "CHAM_PIC_VGA_DRV_V010.h"            // needed for VGA driver
#include "CHAM_PIC_SOUND_DRV_V010.h"          // needed for sound driver

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES AND MACROS /////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// number of notes in scale
#define  NUM_NOTES 13

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// reference for note frequencies
// http://www.phy.mtu.edu/~suits/notefreqs.html

float adjacent_note = 1.059463094; // 2^(1/12) ratio between adjacent keys

// scale from middle A to one octave above A
int scale[] = {440, 466, 494, 523, 554, 587, 622, 659, 698, 740, 784, 831, 880 };

/*
A 	    440
B flat 	466
B    	494
C    	523
C sharp	554
D    	587
D sharp	622
E    	659
F 	    698
F sharp 740
G    	784
A flat 	831
A   	880
*/

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MAIN ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(void)
{
int note;                                           // looping var
float chord_freq_1, chord_freq_2, chord_freq_3;     // hold the computed chord frequencies
char sbuffer[64];                                   // string buffer

// Always call ConfigureClock first
SYS_ConfigureClock(MAX_FCY_RATE);

// initialize SPI
SPI_Init();

// allow Propeller driver time to boot, this is necessary before sending commands, assuming a system reset at the same time
DELAY_MS( 2500 );

// clear screens
NTSC_ClearScreen();
VGA_ClearScreen();

// set colors
NTSC_Color(0);
VGA_Color(0);

// enter infinite loop...
while(1)
     {
     NTSC_Term_Print("Going up...");
     NTSC_Term_Char( 0x0D );

     // play the scale up
     for (note = 0; note < NUM_NOTES; note++)
        {
        // compute chord to play
        chord_freq_1 = scale[ note ]/2 * pow ( adjacent_note, 4);

        // play these an octave above
        chord_freq_2 = scale[ note ] * pow ( adjacent_note, 5);
        chord_freq_3 = scale[ note ] * pow ( adjacent_note, 6);

        sprintf(sbuffer,"Playing Chord [%d, %d, %d]", (int)chord_freq_1, (int)chord_freq_2, (int)chord_freq_3 );

        // play the notes (we have a freq limit of 2049 hz with the current API interface, the driver can do whatever you want though
        // so you will have to create new messages and a 16-bit format if you want to play higher frequencies
        Sound_Play(0, (int)chord_freq_1, 1);
        Sound_Play(1, (int)chord_freq_2, 1);
        Sound_Play(2, (int)chord_freq_3, 1);

        // print on NTSC terminal screen
        NTSC_Term_Print(sbuffer);
        NTSC_Term_Char( 0x0D );

        // print on VGA terminal screen
        VGA_Term_Print(sbuffer);
        VGA_Term_Char( 0x0D );

        // slow things down a bit, so we can read the text!
         DELAY_MS(500);

        } // end for index

     NTSC_Term_Print("Going down...");
     NTSC_Term_Char( 0x0D );

     // play the scale down
     for (note = NUM_NOTES-1; note > 0; note--)
        {
        // compute chord to play
        chord_freq_1 = scale[ note ]/2 * pow ( adjacent_note, 4);

        // play these an octave above
        chord_freq_2 = scale[ note ] * pow ( adjacent_note, 5);
        chord_freq_3 = scale[ note ] * pow ( adjacent_note, 6);

        sprintf(sbuffer,"Playing Chord [%d, %d, %d]", (int)chord_freq_1, (int)chord_freq_2, (int)chord_freq_3 );

        // play the notes (we have a freq limit of 2049 hz with the current API interface, the driver can do whatever you want though
        // so you will have to create new messages and a 16-bit format if you want to play higher frequencies
        Sound_Play(0, (int)chord_freq_1, 1);
        Sound_Play(1, (int)chord_freq_2, 1);
        Sound_Play(2, (int)chord_freq_3, 1);

        // print on NTSC terminal screen
        NTSC_Term_Print(sbuffer);
        NTSC_Term_Char( 0x0D );

        // print on VGA terminal screen
        VGA_Term_Print(sbuffer);
        VGA_Term_Char( 0x0D );

        // slow things down a bit, so we can read the text!
         DELAY_MS(500);

        } // end for index

     } // end while

} // end main
