///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Copyright Nurve Networks LLC 2009
//
// Filename: CHAM_PIC_STATUS_LED_DEMO_01.c
//
// Original Author: Andre' LaMothe
//
// Last Modified: 10.20.09
//
// Description: Demo of controlling the status LED with the modified Propeller Driver
//
// Overview: In the manual there is a section that covers how to modify the Propeller master control program/driver
// to add features and commands, section "oping Your Own Propeller Drivers". As an example, we go into the propeller
// driver and add two commands:
//
//  ' added status LED commands added for example in manual "Developing Your Own Propeller Drivers".
//  STATUS_LED_ON                = 100
//  STATUS_LED_OFF               = 101
//
// and then this little chunk of code in the main message dispatcher:
//
// ' // Propeller status LED commands book example on how to modify the driver
// STATUS_LED_ON:
//  OUTA[25] := 1
//
// STATUS_LED_OFF:
//  OUTA[25] := 0
// ' // end added code for status LED control
//
// This C program then uses these new commands to control the Status LED by sending SPI commands directly.
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// include headers required for demo
#include "CHAM_PIC_SYSTEM_V010.h"             // you need this one always
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"        // you need this always
#include "CHAM_PIC_NTSC_DRV_V010.h"           // needed for NTSC driver

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES AND MACROS /////////////////////////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// new commands for status LED control
#define STATUS_LED_ON  100
#define STATUS_LED_OFF 101

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MAIN ///////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int main(void)
{
// Always call ConfigureClock first
SYS_ConfigureClock(MAX_FCY_RATE);

// initialize SPI
SPI_Init();

// give Prop a moment before sending it commands, the boot process is about a second
// if you like you can speed it up, by removing the Prop driver's "LED blink" sequence in the driver
// or speed it up, but commands that are sent before the Prop driver is done booting will be ignored.
// therefore, if you have a lot of set up work to do, then you don't require a delay, but if you jump right
// into commands, then you need a good 1.5 - 2 second delay
DELAY_MS( 2500 );

// clear screens
NTSC_ClearScreen();
NTSC_Color(0);

// enter infinite loop...
while(1)
     {
     // print on NTSC terminal screen
     NTSC_Term_Print(" Status LED ON");
     NTSC_Term_Char( 0x0D );

     // send command to turn on LED
     SPI_Prop_Send_Cmd(STATUS_LED_ON, 0,0 );

     // slow things down a bit, so we can read the text!
     DELAY_MS(250);

     NTSC_Term_Print(" Status LED OFF");
     NTSC_Term_Char( 0x0D );

     // send command to turn off LED
     SPI_Prop_Send_Cmd(STATUS_LED_OFF, 0,0 );

     // slow things down a bit, so we can read the text!
     DELAY_MS(250);

     } // end while

} // end main
