///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// 
// Copyright Nurve Networks LLC 2009
// 
// Filename: CHAM_PIC_NTSC_DRV_V010.C
//
// Original Author: Andre' LaMothe
//                  Joshua Hintze
// 
// Last Modified: 8.28.09
//
// Description: NTSC API for standard "terminal" mode operations
//  
// Overview:
//
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES ///////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// include local Chameleon PIC API header files
#include "CHAM_PIC_SYSTEM_V010.h"
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"
#include "CHAM_PIC_NTSC_DRV_V010.h"


///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MACROS /////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES/////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// TYPES/CLASSES //////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS ////////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// EXTERNALS //////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PROTOTYPES /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS  /////////////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// NTSC terminal support
/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int NTSC_Term_Print(char *string)
{
// this prints a string to the NTSC terminal (all drivers Default1,2, etc.)
SPI_Prop_Print_String(DEVICE_NTSC, string);

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end NTSC_Term_Print

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int NTSC_Term_Char(char ch)
{
// this prints a single character to the NTSC terminal (all drivers Default1,2, etc.)
// also supports translation commands supported by the NTSC terminals 
//     $00 = clear screen
//     $01 = home
//     $08 = backspace
//     $09 = tab (8 spaces per)
//     $0A = set X position (X follows)
//     $0B = set Y position (Y follows)
//     $0C = set color (color follows)
//     $0D = return
// other characters are not translated, but simply printed to the terminal

SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, ch, 0x00);

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end NTSC_Term_Char

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int NTSC_SetXY(int x, int y)
{
// sets the x,y (column, row) position of the terminal cursor
// TODO: might needs some delays between commands

// send driver set x command, [$0A, x 0..39]           
SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, 0x0A, 0x00);
//DELAY_US(SPI_PROP_DELAY_SHORT_US);
SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, x, 0x00);
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// send driver set x command, [$0B, y 0..29]                           
SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, 0x0B, 0x00);
//DELAY_US(SPI_PROP_DELAY_SHORT_US);
SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, y, 0x00);
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end NTSC_SetXY

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int NTSC_GetXY(int *x, int *y)
{
// gets the x,y (column, row) of the terminal cursor

// send driver get x command
SPI_Prop_Send_Cmd( GFX_CMD_NTSC_GETX, 0x00, 0x00);

//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now read the data back, note 0x00, 0x00 are dummy data, mask lower 8-bits
*x = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00) & 0xFF;

//DELAY_US(SPI_PROP_DELAY_LONG_US);

SPI_Prop_Send_Cmd( GFX_CMD_NTSC_GETY, 0x00, 0x00);

//DELAY_US(SPI_PROP_DELAY_LONG_US);

// now read the data back, note 0x00, 0x00 are dummy data, mask lower 8-bits
*y = SPI_Prop_Send_Cmd( READ_CMD, 0x00, 0x00) & 0xFF;

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end NTSC_GetXY

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int NTSC_ClearScreen(void)
{
// clears the screen of the terminal and fills with spaces

SPI_Prop_Send_Cmd( GFX_CMD_NTSC_CLS, 0x00, 0x00);  
//DELAY_MS(1);

// return success
return(1);

} // end NTSC_ClearScreen

/////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

int NTSC_Color(int col)
{
// sets the "color" of the character, means different things under different drivers
            
// send driver set color command, [$0C, col 0..7]                           
SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, 0x0C, 0x00);
//DELAY_US(SPI_PROP_DELAY_SHORT_US);
SPI_Prop_Send_Cmd( GFX_CMD_NTSC_PRINTCHAR, col & 0x07, 0x00);

// wait a bit, driver takes time to respond...
//DELAY_US(SPI_PROP_DELAY_LONG_US);

// return success
return(1);

} // end NTSC_Color

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
