///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Crate It - Crate pushing fun
// JT Cook - www.avalondreams.com
//
// original wharehouse keeper game and levels by Hiroyuki Imabayashi
// 
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// INCLUDES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>


// Include main system header
#include "CHAM_PIC_SYSTEM_V010.h"

// Include spi driver
#include "CHAM_PIC_I2C_SPI_DRV_V010.h"
//I/O drivers to interface with the Propellor
#include "CHAM_PIC_NTSC_DRV_V010.h"
#include "CHAM_PIC_SOUND_DRV_V010.h"
#include "CHAM_PIC_KEYBOARD_DRV_V010.h"
//game specific files
#include "ci_levels_v010.h" //game levels
#include "ci_gfx_v010.h" //tile graphics
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// DEFINES
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//define arrow keys
#define key_up 194
#define key_down 195
#define key_left 192
#define key_right 193
#define key_space 32
#define key_escape 203
#define key_return 13
//game world
#define map_size_x 20
#define map_size_y 20
#define y_offset 2
//icons
#define floor_icon ' '
#define player_icon (141+34-32)
#define player_wave1_icon (141+34-16)
#define player_wave2_icon (141+34)
#define crate_icon (96+32)
#define wall_icon (102+34)
#define rest_icon (140+34)
//palette for each item
#define player_pal 1
#define wall_pal 2
#define crate_pal 3
#define rest_pal 4
#define floor_pal 4

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// GLOBALS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned char game_map[map_size_x*map_size_y];
unsigned short int player_xy; //current player location
unsigned short int player_moves; //how many moves the player used
unsigned short int player_pushes; //how many crate moves the player used
unsigned short int player_timer; //clock that is used to have player wave
unsigned char current_level; //current level in game
unsigned char player_tile; // tile under the character
unsigned char game_state; //game state (intro screen, in game, etc)
unsigned char player_wave=0;

//addresses for accessing the GPU functions of the Propellor
short int tilemap_adr;
short int tilegfx_adr;
short int palette_adr;

//matches tile up with game map
unsigned const char gfx_map[]= {floor_icon, wall_icon, crate_icon, rest_icon, crate_icon, floor_icon};
unsigned const char pal_map[]= {floor_pal, wall_pal, crate_pal, rest_pal, crate_pal, 5};

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// FUNCTIONS
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
void setup_game_gfx();
void game_loop();
void draw_intro_screen();
void draw_player(unsigned char char_num);
void draw_map();
void check_map();
void load_map(unsigned char map_level);
void move_player(unsigned char direction);
void draw_string(unsigned char x, unsigned char y, unsigned char *string_data);
void draw_moves(unsigned short int add_move);
void draw_pushes(unsigned short int add_move);
void write_prop_byte(unsigned int adr, unsigned char byte);
void write_prop_int(unsigned int adr, unsigned short int byte);
void draw_tile(int x, int y, unsigned char data, unsigned char palette);
void play_sfx(unsigned char sound_num);
void do_finish();
// Our main start location
int main (void)
{
	int PreviousPortValue = 0;
	volatile gid_event event;  // generic "event" for mouse, etc.

	// Always call ConfigureClock first
	SYS_ConfigureClock(MAX_FCY_RATE);

	// Initialize the spi interface
	SPI_SET_CS(SPI_CS_0);
	SPI_Init();

	// Wait for the propeller to initialize
	DELAY_SEC(3);

	// Clear screen
	NTSC_ClearScreen();

	//this should already be started
	Keyboard_Load();
	
	//load game tiles into the Propellor
    setup_game_gfx();

	//set top and bottom overscan colors
    GFX_Set_Top_Overscan_Color(0x02);
    GFX_Set_Bottom_Overscan_Color(0x02);

    //start program
    draw_intro_screen();
    game_state=0;
    int key;

    //enter main loop
	while(1)
	{
     switch(game_state)
	 {
	   case 0: //intro screen
	     //wait for player to hit enter
		key = Keyboard_Key();
		if(key==key_return)
		{
		  game_state=1;
		  current_level=0;
		  load_map(current_level);
		}
	   break;
	   case 1: //in game
		game_loop();
	   break;
	   case 2: //finish screen
	    //wait for player to hit space
		key = Keyboard_Key();
		if(key==key_space)
		{
    	 draw_intro_screen();
    	 game_state=0;
		}
	   break;

	 }
	}
}

///////////////////////////////////////////
// copy graphics over
void setup_game_gfx()
{
	short int temp_adr,n,n2,n3=0;
	unsigned char temp_color;
    //location of characters in memory
	unsigned const char c_map[]={player_icon, wall_icon, crate_icon,rest_icon, player_wave1_icon, player_wave2_icon};
	//get the addresses on the Prop for gfx functions
    tilemap_adr=GFX_Read_Tilemap_Ptr();
    tilegfx_adr=GFX_Read_Bitmap_Ptr();
	palette_adr=GFX_Read_Palette_Ptr();
    
	//write new tile gfx
	for(n=0;n<6;n++)
	{
	  temp_adr=tilegfx_adr+((c_map[n]-32)<<4);
	  for(n2=0;n2<16;n2++)
	  {
		write_prop_byte(temp_adr+n2,s_gfx[n3]);
		n3++; 
	  } 
	}

    //write new palette
	temp_adr=palette_adr+4;
	n3=0;
	//write new palette
	for(n=0;n<5;n++)
	{
	 for(n2=0;n2<4;n2++)
	 {
	  temp_color=s_pal[n3];
	  write_prop_byte(temp_adr+n3,temp_color);
	  n3++;
	 }
	}
	//write 3rd palette to first entry
	n3=12;
	temp_adr=palette_adr;
	for(n=0;n<4;n++)
	{
	 write_prop_byte(temp_adr+n,s_pal[n3]);
	 n3++;
	}

}
///////////////////////////////////////////
// game loop
void game_loop()
{
	// Keyboard test
		// Echo keyboard back to NTSC screen
		int Key = Keyboard_Key();
		if(Key != 0)
		{
		 player_timer=0;
		 switch(Key)
		 {
			case key_up:
             move_player(0);
			break;
			case key_down:
             move_player(1);
			break;
			case key_left:
             move_player(2);
			break;
			case key_right:
             move_player(3);
			break;
			//cheat to move through the levels freely
			case '+':
			 if(current_level<max_level-1)
			 {
              current_level++;
			  load_map(current_level);
			 }
			break;
			case '-':
			 if(current_level>0)
			 {
              current_level--;
			  load_map(current_level);
			 }
			break;
			//restart map
			case key_escape:
			 load_map(current_level);
			break;
		 }
		}
		//no key hit
		else
		{
		  DELAY_MS(50);
		  player_timer++;
		  if(player_timer>150)
		  {
			player_timer-=5;
			draw_player(player_wave+1);
			player_wave^=1;
		  }
		}
}
////////////////////////////////////////////////
// Intro screen
void draw_intro_screen()
{
  //24x11 banner
  unsigned char x,y;
  unsigned short int n=0;
  NTSC_ClearScreen();
  //draw start screen
  for(y=0;y<11;y++)
  {
    for(x=0;x<24;x++)
    {
	    if(intro_screen_gfx[n]==1)
		{
			draw_tile(x+3,y+4,(wall_icon-32),wall_pal);
		}
		n++;
    }
  }
  draw_string(11,17,"Hit Enter");
  draw_string(5,19,  "www.chameleon-dev.com");
  draw_string(5,20,  "www.xgamestation.com");
  draw_string(12,21,  "JT Cook");
  draw_string(0, 23, "Levels by: Hiroyuki Imabayashi");     
}
////////////////////////////////////////////////
// draws player
void draw_player(unsigned char char_num)
{
 unsigned short int x,y,n;
 char_num<<=4;
 //get x/y position
 y=player_xy/map_size_x;
 x=player_xy-(y*map_size_x);
 draw_tile(x, y+y_offset,player_icon-32+char_num,player_pal);
}

////////////////////////////////////////////////
//draw moves
void draw_moves(unsigned short int add_move)
{
 char text_data[20];
    //update number of moves
    player_moves+=add_move;
	sprintf(text_data,"%d",player_moves);
	//draw_string(6,22,text_data);
    draw_string(27,8,text_data);
}
////////////////////////////////////////////////
//draw pushes
void draw_pushes(unsigned short int add_move)
{
 char text_data[20];
    //update number of moves
    player_pushes+=add_move;
	sprintf(text_data,"%d",player_pushes);
	//draw_string(6,22,text_data);
    draw_string(28,10,text_data);
}

////////////////////////////////////////////////
// Draws the game board
void draw_board()
{
  char text_data[30];
  draw_string(22,2,"Crate-It");
  sprintf(text_data,"Level:%d",current_level+1);
  draw_string(21,6,text_data);
  draw_string(21,8,"Moves:0");
  draw_string(21,10,"Pushes:0");  
  //sprintf(text_data,"Moves:0");
  //draw_string(0,22,text_data);
  //sprintf(text_data,"Level:%d",current_level+1);
  //draw_string(0,23,text_data);
  draw_string(21,15,"Arrow keys-");
  draw_string(21,16,"Move guy");
  draw_string(21,18,"Esc-Reset");
  draw_string(25,19,"Level");
  draw_string(21,21,"Chameleon");
  draw_string(21,22,"16-bit PIC");

}

////////////////////////////////////////////////
// Draws the game map
void draw_map()
{
 unsigned char x,y;
 unsigned short int n=0;
 NTSC_ClearScreen();
 for(y=0;y<map_size_y;y++)
 {
  for(x=0;x<map_size_x;x++)
  {
    //handle the info in the game world
    draw_tile(x, y+y_offset,gfx_map[game_map[n]]-32,pal_map[game_map[n]]);
    n++;
  }
 }
 draw_board();
 draw_player(0);
}
////////////////////////////////////////////////
// Checks the map to see if all the crates are in place
void check_map()
{
 unsigned short int n, Key=0;
 for(n=0;n<(map_size_x*map_size_y);n++)
 {
  //if there is still an open rest area, then game is not done
  if(game_map[n]==3)
   return;
 }
 //map is complete, wait for reponse to go to next level
 //draw_string(0,23,"Level complete! Hit Enter");
 draw_string(3,20,"Level complete!");
 draw_string(6,21,"Hit Enter");
 play_sfx(4);
 //wait for player to hit enter
 while(Key!=key_return)
 {
	Key = Keyboard_Key();
 }
 //if the map is done, advance to the next level
 if(current_level<max_level-1)
 {
	current_level++;
    load_map(current_level);
 }
 //end screen
 else
 {
  game_state=2;
  do_finish();
 }
}

////////////////////////////////////////////////
// Move player
void move_player(unsigned char direction)
{
 unsigned short int new_xy, crate_xy, x,y;
 //directions 0-up, 1-down, 2-left, 3-right
 switch(direction)
 {
	//up
	case 0:
	 new_xy=player_xy-map_size_x;
	 crate_xy=new_xy-map_size_x;
	break;
	//down
	case 1:
	 new_xy=player_xy+map_size_x;
	 crate_xy=new_xy+map_size_x;
	break;
	//left
	case 2:
	 new_xy=player_xy-1;
	 crate_xy=new_xy-1;
	break;
	//right
	case 3:
	 new_xy=player_xy+1;
	 crate_xy=new_xy+1;
	break;
 }

 //floor or crate resting area
 if(game_map[new_xy]==0 || game_map[new_xy]==3)
 {
    y=player_xy/map_size_x;
    x=player_xy-(y*map_size_x);
    //erase old player
    draw_tile(x,y+y_offset,gfx_map[game_map[player_xy]]-32,pal_map[game_map[player_xy]]);

    //get new location and draw player
    player_xy=new_xy;
	draw_player(0);
    //update number of moves
	draw_moves(1);
    play_sfx(1);
 }

 //pushing a crate
 else if(game_map[new_xy]==2 || game_map[new_xy]==4)
 {
   //check to see if there is room infront of crate to push
   if(game_map[crate_xy]==0 || game_map[crate_xy]==3)
   {
    //erase old player
    y=player_xy/map_size_x;
    x=player_xy-(y*map_size_x);
    draw_tile(x,y+y_offset,gfx_map[game_map[player_xy]]-32,pal_map[game_map[player_xy]]);

    //get new location and draw player
    player_xy=new_xy;

    //if we are pushing a crate in a rest area, put rest area back
    if(game_map[player_xy]==4)
       game_map[player_xy]=3;
    //otherwise remove the crate
    else
       game_map[player_xy]=0;

	draw_player(0);
    //draw crate
    y=crate_xy/map_size_x;
    x=crate_xy-(y*map_size_x);
    draw_tile(x,y+y_offset,crate_icon-32,crate_pal);

    //update floor tile since it should now hold the crate
    if(game_map[crate_xy]==0)
	 game_map[crate_xy]=2;

    //if crate rest area, then update with crate on rest area
    if(game_map[crate_xy]==3)
	 game_map[crate_xy]=4;

    //update number of moves
	draw_moves(1);
	draw_pushes(1);
    play_sfx(3);
    //check to see if we have completed the level
    check_map();
   }
   //can't move crate
   else
    play_sfx(2);    
 }
 //hits a wall
 else
   play_sfx(2);
}

////////////////////////////////////////////////
// loads a map, draws it on the screen
void load_map(unsigned char map_level)
{
 unsigned short int n, map_offset;
 player_moves=0; //reset move count
 player_pushes=0;
 player_timer=0;
 map_offset=map_level*(map_size_x*map_size_y);
 for(n=0;n<(map_size_x*map_size_y);n++)
 {
    game_map[n]=world_1[n+map_offset];
    if(game_map[n]==9)
	{
       player_xy=n;
	   player_tile=0;
	   game_map[n]=0; //place a floor tile
    }
 }
 draw_map();
}

////////////////////////////////////////////////
// draw a string
void draw_string(unsigned char x, unsigned char y, unsigned char *string_data)
{
 unsigned char n;
 //make sure we aren't looping forever
 for(n=0;n<40;n++)
 {
  if(string_data[n]!=0)
   draw_tile(x+n,y,string_data[n]-32,0);
  else
   return;
 }
}
///////////////////////////////////////////////////
// plays a sound
void play_sfx(unsigned char sound_num)
{
 unsigned short int sound_pitch, n;
 switch(sound_num)
 {
  //stop sound
  case 0:
			Sound_Stop(0);
  break;
  //walk sound
  case 1:
			// play a sound
			Sound_Play(0, 600, 0);
			DELAY_MS(10);
			Sound_Stop(0);
  break;
  //wall sound
  case 2:
			// play a sound
			Sound_Play(0, 250, 0);
			DELAY_MS(50);
			Sound_Stop(0);
  break;
  //crate push sound
  case 3:
			// play a sound
			Sound_Play(0, 250, 0);
			DELAY_MS(10);
			Sound_Stop(0);
			Sound_Play(0, 400, 0);
			DELAY_MS(10);
			Sound_Stop(0);
			Sound_Play(0, 250, 0);
			DELAY_MS(10);
			Sound_Stop(0);
  break;
  //finish level sound
  case 4:
    sound_pitch=1200;
   	Sound_Play(0, sound_pitch, 0);
    for(n=0;n<25;n++)
    {
     DELAY_MS(15);
     sound_pitch-=25;
     Sound_Update(0, sound_pitch, 0);
    }
    for(n=0;n<25;n++)
    {
     DELAY_MS(15);
     sound_pitch+=25;
     Sound_Update(0, sound_pitch, 0);
    }
	DELAY_MS(15);
	Sound_Stop(0);
  break;
 }
}
///////////////////////////////////////////////////
// writes a byte to the prop
void write_prop_byte(unsigned int adr, unsigned char byte) 
{
    GFX_Set_Src_Ptr(adr);
    GFX_Write_Mem_Port8(byte);
}

///////////////////////////////////////////////////
// writes a int to the prop
void write_prop_int(unsigned int adr, unsigned short int byte) 
{
    GFX_Set_Src_Ptr(adr);
    GFX_Write_Mem_Port16(byte);
}

///////////////////////////////////////////////////
// draws a character on the screen
void draw_tile(int x, int y, unsigned char data, unsigned char palette)
{
  short int offset;
  offset=tilemap_adr + ((y<<6)+(x<<1));
  write_prop_int(offset,(palette<<8)|data);
}

////////////////////////////////////////////////
//finished all the mazes
void do_finish()
{
	NTSC_ClearScreen();
    //              "................................"
    draw_string(0,1,"Crate-It wants to congratulate");
    draw_string(0,2,"you for passing all the levels!");
    draw_string(0,4,"After this display of Einstein");
    draw_string(0,5,"level of intelligence you could");
    draw_string(0,6,"consider joining Mensa. Or if");
    draw_string(0,7,"you are feeling creative you");
    draw_string(0,8,"should consider making your own");
    draw_string(0,9,"Chameleon program!");
    
    draw_string(0,11,"If you feel like you have some");
    draw_string(0,12,"unfinished business left in the");
    draw_string(0,13,"game, we understand. I mean I");
    draw_string(0,14,"saw the number of moves you had");
    draw_string(0,15,"in some of levels, surely you");
    draw_string(0,16,"can do better than that!");

	draw_string(2,19,"Hit space to continue...");
}
